#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re

from tempest_lib import exceptions

from saharaclient.tests.functional import base


class SimpleReadOnlySaharaClientTest(base.ClientTestBase):
    """Basic, read-only tests for Sahara CLI client.

    Checks return values and output of read-only commands.
    These tests do not presume any content, nor do they create
    their own. They only verify the structure of output if present.
    """

    def test_sahara_fake_action(self):
        self.assertRaises(exceptions.CommandFailed,
                          self.sahara,
                          'this-does-not-exist')

    def test_sahara_plugins_list(self):
        plugins = self.parser.listing(self.sahara('plugin-list'))
        self.assertTableStruct(plugins, [
            'name',
            'versions',
            'title'
        ])

    def test_sahara_plugins_show(self):
        plugins = self.parser.listing(self.sahara('plugin-list'))
        plugin_names = [p['name'] for p in plugins]
        if not plugin_names:
            raise self.skipException('No plugins defined')
        name_param = '--name %s' % (plugin_names[0])
        result = self.sahara('plugin-show', params=name_param)
        plugin = self.parser.listing(result)
        self.assertTableStruct(plugin, [
            'Property',
            'Value'
        ])

    def test_sahara_node_group_template_list(self):
        result = self.sahara('node-group-template-list')
        node_group_templates = self.parser.listing(result)
        self.assertTableStruct(node_group_templates, [
            'name',
            'id',
            'plugin_name',
            'node_processes',
            'description'
        ])

    def test_sahara_cluster_template_list(self):
        result = self.sahara('cluster-template-list')
        cluster_templates = self.parser.listing(result)
        self.assertTableStruct(cluster_templates, [
            'name',
            'id',
            'plugin_name',
            'node_groups',
            'description'
        ])

    def test_sahara_cluster_list(self):
        result = self.sahara('cluster-list')
        clusters = self.parser.listing(result)
        self.assertTableStruct(clusters, [
            'name',
            'id',
            'status',
            'node_count'
        ])

    def test_sahara_data_source_list(self):
        result = self.sahara('data-source-list')
        data_sources = self.parser.listing(result)
        self.assertTableStruct(data_sources, [
            'name',
            'id',
            'type',
            'description'
        ])

    def test_sahara_job_binary_data_list(self):
        result = self.sahara('job-binary-data-list')
        job_binary_data_list = self.parser.listing(result)
        self.assertTableStruct(job_binary_data_list, [
            'id',
            'name'
        ])

    def test_sahara_job_binary_list(self):
        result = self.sahara('job-binary-list')
        job_binaries = self.parser.listing(result)
        self.assertTableStruct(job_binaries, [
            'id',
            'name',
            'description'
        ])

    def test_sahara_job_template_list(self):
        result = self.sahara('job-template-list')
        job_templates = self.parser.listing(result)
        self.assertTableStruct(job_templates, [
            'id',
            'name',
            'description'
        ])

    def test_sahara_job_list(self):
        result = self.sahara('job-list')
        jobs = self.parser.listing(result)
        self.assertTableStruct(jobs, [
            'id',
            'cluster_id',
            'status'
        ])

    def test_sahara_bash_completion(self):
        self.sahara('bash-completion')

    # Optional arguments
    def test_sahara_help(self):
        help_text = self.sahara('help')
        lines = help_text.split('\n')
        self.assertIn('DeprecationWarning', lines[0])

        commands = []
        cmds_start = lines.index('Positional arguments:')
        cmds_end = lines.index('Optional arguments:')
        command_pattern = re.compile('^ {4}([a-z0-9\-\_]+)')
        for line in lines[cmds_start:cmds_end]:
            match = command_pattern.match(line)
            if match:
                commands.append(match.group(1))
        commands = set(commands)
        wanted_commands = set(('cluster-create', 'data-source-create',
                               'image-unregister', 'job-binary-create',
                               'plugin-list', 'job-binary-create', 'help'))
        self.assertFalse(wanted_commands - commands)

    def test_sahara_version(self):
        version = self.sahara('', flags='--version')
        self.assertTrue(re.search('[0-9.]+', version))



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re
import sys

import fixtures
import mock
import six
from testtools import matchers

import saharaclient.api.client
from saharaclient.openstack.common.apiclient import exceptions
import saharaclient.shell
from saharaclient.tests.unit.nova import utils

FAKE_ENV = {'OS_USERNAME': 'username',
            'OS_PASSWORD': 'password',
            'OS_TENANT_NAME': 'tenant_name',
            'OS_AUTH_URL': 'http://no.where'}

FAKE_ENV2 = {'OS_USERNAME': 'username',
             'OS_PASSWORD': 'password',
             'OS_TENANT_ID': 'tenant_id',
             'OS_AUTH_URL': 'http://no.where'}


class FakePlugin(object):
    name = 'fake'
    versions = ['1.0', ]
    title = 'a fake plugin'


class FakePluginManager(object):
    def list(self):
        return (FakePlugin(),)


class FakeImage(object):
    name = 'fake'
    id = 'aaa-bb-ccc'
    username = 'you'
    description = None
    tags = []


class FakeImageManager(object):
    def list(self):
        return (FakeImage(),)


class FakePluginClient(object):
    def __init__(self, *args, **kwargs):
        self.plugins = FakePluginManager()
        self.images = FakeImageManager()


class ShellTest(utils.TestCase):

    def make_env(self, exclude=None, fake_env=FAKE_ENV):
        env = dict((k, v) for k, v in fake_env.items() if k != exclude)
        self.useFixture(fixtures.MonkeyPatch('os.environ', env))

    def setUp(self):
        super(ShellTest, self).setUp()
# NA atm
#       self.useFixture(fixtures.MonkeyPatch(
#                       'novaclient.client.get_client_class',
#                       mock.MagicMock))
#       self.nc_util = mock.patch('novaclient.utils.isunauthenticated').start()
#       self.nc_util.return_value = False

    def shell(self, argstr, exitcodes=(0,)):
        orig = sys.stdout
        orig_stderr = sys.stderr
        try:
            sys.stdout = six.StringIO()
            sys.stderr = six.StringIO()
            _shell = saharaclient.shell.OpenStackSaharaShell()
            _shell.main(argstr.split())
        except SystemExit:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            self.assertIn(exc_value.code, exitcodes)
        finally:
            stdout = sys.stdout.getvalue()
            sys.stdout.close()
            sys.stdout = orig
            stderr = sys.stderr.getvalue()
            sys.stderr.close()
            sys.stderr = orig_stderr
        return (stdout, stderr)

    def test_help_unknown_command(self):
        self.assertRaises(exceptions.CommandError, self.shell, 'help foofoo')

# NA
#    def test_invalid_timeout(self):
#        for f in [0, -1, -10]:
#            cmd_text = '--timeout %s' % (f)
#            stdout, stderr = self.shell(cmd_text, exitcodes=[0, 2])
#            required = [
#                'argument --timeout: %s must be greater than 0' % (f),
#            ]
#            for r in required:
#                self.assertIn(r, stderr)

    def test_help(self):
        required = [
            '.*?^usage: sahara',
            '.*?^\s+plugin-list\s+Print a list of available plugins.',
            '.*?^See "sahara help COMMAND" for help on a specific command',
        ]
        stdout, stderr = self.shell('help')
        for r in required:
            self.assertThat((stdout + stderr),
                            matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))

    def test_help_on_subcommand(self):
        required = [
            '.*?^usage: sahara plugin-list',
            '.*?^Print a list of available plugins.',
        ]
        stdout, stderr = self.shell('help plugin-list')
        for r in required:
            self.assertThat((stdout + stderr),
                            matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))

    def test_help_no_options(self):
        required = [
            '.*?^usage: sahara',
            '.*?^\s+plugin-list\s+Print a list of available plugins.',
            '.*?^See "sahara help COMMAND" for help on a specific command',
        ]
        stdout, stderr = self.shell('')
        for r in required:
            self.assertThat((stdout + stderr),
                            matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))

    def test_bash_completion(self):
        stdout, stderr = self.shell('bash-completion')
        # just check we have some output
        required = [
            '.*help',
            '.*plugin-list',
            '.*plugin-show',
            '.*--name']
        for r in required:
            self.assertThat((stdout + stderr),
                            matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))

    def test_no_username(self):
        required = ('You must provide a username'
                    ' via either --os-username or env[OS_USERNAME]',)
        self.make_env(exclude='OS_USERNAME')
        try:
            self.shell('plugin-list')
        except exceptions.CommandError as message:
            self.assertEqual(required, message.args)
        else:
            self.fail('CommandError not raised')

    def test_no_tenant_name(self):
        required = (
            'You must provide a tenant_name, tenant_id, '
            'project_id or project_name (with '
            'project_domain_name or project_domain_id) via '
            '  --os-tenant-name (env[OS_TENANT_NAME]),'
            '  --os-tenant-id (env[OS_TENANT_ID]),'
            '  --os-project-id (env[OS_PROJECT_ID])'
            '  --os-project-name (env[OS_PROJECT_NAME]),'
            '  --os-project-domain-id '
            '(env[OS_PROJECT_DOMAIN_ID])'
            '  --os-project-domain-name '
            '(env[OS_PROJECT_DOMAIN_NAME])',
        )
        self.make_env(exclude='OS_TENANT_NAME')
        try:
            self.shell('plugin-list')
        except exceptions.CommandError as message:
            self.assertEqual(required, message.args)
        else:
            self.fail('CommandError not raised')

    def test_no_tenant_id(self):
        required = (
            'You must provide a tenant_name, tenant_id, '
            'project_id or project_name (with '
            'project_domain_name or project_domain_id) via '
            '  --os-tenant-name (env[OS_TENANT_NAME]),'
            '  --os-tenant-id (env[OS_TENANT_ID]),'
            '  --os-project-id (env[OS_PROJECT_ID])'
            '  --os-project-name (env[OS_PROJECT_NAME]),'
            '  --os-project-domain-id '
            '(env[OS_PROJECT_DOMAIN_ID])'
            '  --os-project-domain-name '
            '(env[OS_PROJECT_DOMAIN_NAME])',
        )
        self.make_env(exclude='OS_TENANT_ID', fake_env=FAKE_ENV2)
        try:
            self.shell('plugin-list')
        except exceptions.CommandError as message:
            self.assertEqual(required, message.args)
        else:
            self.fail('CommandError not raised')

    def test_no_auth_url(self):
        required = ('You must provide an auth url'
                    ' via either --os-auth-url or env[OS_AUTH_URL] or'
                    ' specify an auth_system which defines a default url'
                    ' with --os-auth-system or env[OS_AUTH_SYSTEM]',)
        self.make_env(exclude='OS_AUTH_URL')
        try:
            self.shell('plugin-list')
        except exceptions.CommandError as message:
            self.assertEqual(required, message.args)
        else:
            self.fail('CommandError not raised')

#    @mock.patch('sys.stdin', side_effect=mock.MagicMock)
#    @mock.patch('getpass.getpass', return_value='password')
#    def test_password(self, mock_getpass, mock_stdin):
    @mock.patch('saharaclient.api.client.Client', FakePluginClient)
    def test_password(self):
        ex = (
            '+------+----------+---------------+\n'
            '| name | versions | title         |\n'
            '+------+----------+---------------+\n'
            '| fake | 1.0      | a fake plugin |\n'
            '+------+----------+---------------+\n'
        )
#        self.make_env(exclude='OS_PASSWORD')
        self.make_env()
        stdout, stderr = self.shell('plugin-list')
        self.assertEqual(ex, (stdout + stderr))

#    @mock.patch('sys.stdin', side_effect=mock.MagicMock)
#    @mock.patch('getpass.getpass', side_effect=EOFError)
#    def test_no_password(self, mock_getpass, mock_stdin):
    def test_no_password(self):
        required = ('Expecting a password provided'
                    ' via either --os-password, env[OS_PASSWORD],'
                    ' or prompted response',)
        self.make_env(exclude='OS_PASSWORD')
        try:
            self.shell('plugin-list')
        except exceptions.CommandError as message:
            self.assertEqual(required, message.args)
        else:
            self.fail('CommandError not raised')

# TODO(mattf) Only one version of API right now
#    def _test_service_type(self, version, service_type, mock_client):
#        if version is None:
#            cmd = 'list'
#        else:
#            cmd = '--os-compute-api-version %s list' % version
#        self.make_env()
#        self.shell(cmd)
#        _, client_kwargs = mock_client.call_args
#        self.assertEqual(service_type, client_kwargs['service_type'])
#
#    @mock.patch('novaclient.client.Client')
#    def test_default_service_type(self, mock_client):
#        self._test_service_type(None, 'compute', mock_client)
#
#    @mock.patch('novaclient.client.Client')
#    def test_v1_1_service_type(self, mock_client):
#        self._test_service_type('1.1', 'compute', mock_client)
#
#    @mock.patch('novaclient.client.Client')
#    def test_v2_service_type(self, mock_client):
#        self._test_service_type('2', 'compute', mock_client)
#
#    @mock.patch('novaclient.client.Client')
#    def test_v3_service_type(self, mock_client):
#        self._test_service_type('3', 'computev3', mock_client)
#
#    @mock.patch('novaclient.client.Client')
#    def test_v_unknown_service_type(self, mock_client):
#        self._test_service_type('unknown', 'compute', mock_client)

    @mock.patch('saharaclient.api.client.Client', FakePluginClient)
    def test_image_list(self):
        ex = (
            '+------+------------+----------+------+-------------+\n'
            '| name | id         | username | tags | description |\n'
            '+------+------------+----------+------+-------------+\n'
            '| fake | aaa-bb-ccc | you      |      | None        |\n'
            '+------+------------+----------+------+-------------+\n'
        )
        self.make_env()
        stdout, stderr = self.shell('image-list')
        self.assertEqual(ex, (stdout + stderr))


class ShellTestKeystoneV3(ShellTest):

    FAKE_V3_ENV = {'OS_USERNAME': 'username',
                   'OS_PASSWORD': 'password',
                   'OS_PROJECT_NAME': 'project_name',
                   'OS_PROJECT_DOMAIN_NAME': 'project_domain_name',
                   'OS_USER_DOMAIN_NAME': 'user_domain_name',
                   'OS_AUTH_URL': 'http://no.where/v3'}

    version_id = u'v3'
    links = [{u'href': u'http://no.where/v3', u'rel': u'self'}]

    def make_env(self, exclude=None, fake_env=FAKE_V3_ENV):
        if 'OS_AUTH_URL' in fake_env:
            fake_env.update({'OS_AUTH_URL': 'http://no.where/v3'})
        env = dict((k, v) for k, v in fake_env.items() if k != exclude)
        self.useFixture(fixtures.MonkeyPatch('os.environ', env))

    def test_no_tenant_name(self):
        # In V3, tenant_name = project_name
        required = (
            'You must provide a tenant_name, tenant_id, '
            'project_id or project_name (with '
            'project_domain_name or project_domain_id) via '
            '  --os-tenant-name (env[OS_TENANT_NAME]),'
            '  --os-tenant-id (env[OS_TENANT_ID]),'
            '  --os-project-id (env[OS_PROJECT_ID])'
            '  --os-project-name (env[OS_PROJECT_NAME]),'
            '  --os-project-domain-id '
            '(env[OS_PROJECT_DOMAIN_ID])'
            '  --os-project-domain-name '
            '(env[OS_PROJECT_DOMAIN_NAME])',
        )
        self.make_env(exclude='OS_PROJECT_NAME')
        try:
            self.shell('plugin-list')
        except exceptions.CommandError as message:
            self.assertEqual(required, message.args)
        else:
            self.fail('CommandError not raised')

    def test_job_list(self):
        expected = '\n'.join([
            '+----+------------+------------+--------+',
            '| id | cluster_id | start_time | status |',
            '+----+------------+------------+--------+',
            '+----+------------+------------+--------+',
            ''
        ])

        mock_get_service_type_method_name = (
            'saharaclient.api.client.Client._determine_service_type')
        mock_job_executions_class_name = (
            'saharaclient.api.job_executions.JobExecutionsManager')

        with mock.patch(mock_get_service_type_method_name) as mock_st:
            with mock.patch(mock_job_executions_class_name):
                mock_st.return_value = 'data-processing'
                self.make_env()
                stdout, stderr = self.shell('job-list')
                self.assertEqual((stdout + stderr), expected)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from saharaclient.api import job_types as api_jt
from saharaclient.api import jobs as api_j
from saharaclient.osc.v1 import job_types as osc_jt

from saharaclient.tests.unit.osc.v1 import fakes

JOB_TYPE_INFO = {
    "name": 'Pig',
    "plugins": [
        {
            'versions': {
                '0.1': {},
                '0.2': {}
            },
            'name': 'fake'
        },
        {
            'versions': {
                '6.2.2': {}
            },
            'name': 'wod'
        }
    ]
}


class TestJobTypes(fakes.TestDataProcessing):
    def setUp(self):
        super(TestJobTypes, self).setUp()
        self.job_mock = self.app.client_manager.data_processing.jobs
        self.jt_mock = self.app.client_manager.data_processing.job_types
        self.jt_mock.reset_mock()
        self.job_mock.reset_mock()


class TestListJobTemplates(TestJobTypes):
    def setUp(self):
        super(TestListJobTemplates, self).setUp()
        self.jt_mock.list.return_value = [api_jt.JobType(None, JOB_TYPE_INFO)]

        # Command to test
        self.cmd = osc_jt.ListJobTypes(self.app, None)

    def test_job_types_list_no_options(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Name', 'Plugins']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [('Pig', 'fake(0.1, 0.2), wod(6.2.2)')]
        self.assertEqual(expected_data, list(data))

    def test_job_types_list_extra_search_opts(self):
        arglist = ['--type', 'Pig', '--plugin', 'fake', '--version', '0.1']
        verifylist = [('type', 'Pig'), ('plugin', 'fake'), ('version', '0.1')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Name', 'Plugins']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [('Pig', 'fake(0.1, 0.2), wod(6.2.2)')]
        self.assertEqual(expected_data, list(data))


class TestGetJobTypeConfigs(TestJobTypes):
    def setUp(self):
        super(TestGetJobTypeConfigs, self).setUp()
        self.job_mock.get_configs.return_value = (
            api_j.Job(None, JOB_TYPE_INFO))

        # Command to test
        self.cmd = osc_jt.GetJobTypeConfigs(self.app, None)

    @mock.patch('oslo_serialization.jsonutils.dump')
    def test_get_job_type_configs_default_file(self, p_dump):
        m_open = mock.mock_open()
        with mock.patch('six.moves.builtins.open', m_open, create=True):
            arglist = ['Pig']
            verifylist = [('job_type', 'Pig')]

            parsed_args = self.check_parser(self.cmd, arglist, verifylist)

            self.cmd.take_action(parsed_args)

            # Check that correct arguments was passed
            self.job_mock.get_configs.assert_called_once_with(
                'Pig')

            args_to_dump = p_dump.call_args[0]
            # Check that the right data will be saved

            self.assertEqual(JOB_TYPE_INFO, args_to_dump[0])
            # Check that data will be saved to the right file
            self.assertEqual('Pig', m_open.call_args[0][0])

    @mock.patch('oslo_serialization.jsonutils.dump')
    def test_get_job_type_configs_specified_file(self, p_dump):
        m_open = mock.mock_open()
        with mock.patch('six.moves.builtins.open', m_open):
            arglist = ['Pig', '--file', 'testfile']
            verifylist = [('job_type', 'Pig'), ('file', 'testfile')]

            parsed_args = self.check_parser(self.cmd, arglist, verifylist)

            self.cmd.take_action(parsed_args)

            # Check that correct arguments was passed
            self.job_mock.get_configs.assert_called_once_with(
                'Pig')

            args_to_dump = p_dump.call_args[0]
            # Check that the right data will be saved

            self.assertEqual(JOB_TYPE_INFO, args_to_dump[0])
            # Check that data will be saved to the right file
            self.assertEqual('testfile', m_open.call_args[0][0])



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

from cliff import command
from cliff import lister
from cliff import show
from openstackclient.common import utils as osc_utils
from oslo_log import log as logging

from saharaclient.osc.v1 import utils

IMAGE_FIELDS = ['name', 'id', 'username', 'tags', 'status', 'description']


class ListImages(lister.Lister):
    """Lists registered images"""

    log = logging.getLogger(__name__ + ".ListImages")

    def get_parser(self, prog_name):
        parser = super(ListImages, self).get_parser(prog_name)
        parser.add_argument(
            '--long',
            action='store_true',
            default=False,
            help='List additional fields in output',
        )
        parser.add_argument(
            '--name',
            metavar="<name-regex>",
            help="Regular expression to match image name"
        )
        parser.add_argument(
            '--tags',
            metavar="<tag>",
            nargs="+",
            help="List images with specific tag(s)"
        )
        parser.add_argument(
            '--username',
            metavar="<username>",
            help="List images with specific username"
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing
        search_opts = {'tags': parsed_args.tags} if parsed_args.tags else {}

        data = client.images.list(search_opts=search_opts)

        if parsed_args.name:
            data = utils.get_by_name_substring(data, parsed_args.name)

        if parsed_args.username:
            data = [i for i in data if parsed_args.username in i.username]

        if parsed_args.long:
            columns = IMAGE_FIELDS
            column_headers = [c.capitalize() for c in columns]

        else:
            columns = ('name', 'id', 'username', 'tags')
            column_headers = [c.capitalize() for c in columns]

        return (
            column_headers,
            (osc_utils.get_item_properties(
                s,
                columns,
                formatters={
                    'tags': osc_utils.format_list
                },
            ) for s in data)
        )


class ShowImage(show.ShowOne):
    """Display image details"""

    log = logging.getLogger(__name__ + ".ShowImage")

    def get_parser(self, prog_name):
        parser = super(ShowImage, self).get_parser(prog_name)
        parser.add_argument(
            "image",
            metavar="<image>",
            help="Name or id of the image to display",
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing

        data = utils.get_resource(
            client.images, parsed_args.image).to_dict()
        data['tags'] = osc_utils.format_list(data['tags'])

        data = utils.prepare_data(data, IMAGE_FIELDS)

        return self.dict2columns(data)


class RegisterImage(show.ShowOne):
    """Register an image"""

    log = logging.getLogger(__name__ + ".RegisterImage")

    def get_parser(self, prog_name):
        parser = super(RegisterImage, self).get_parser(prog_name)
        parser.add_argument(
            "image",
            metavar="<image>",
            help="Name or ID of the image to register",
        )
        parser.add_argument(
            "--username",
            metavar="<username>",
            help="Username of privileged user in the image [REQUIRED]",
            required=True
        )
        parser.add_argument(
            "--description",
            metavar="<description>",
            help="Description of the image. If not provided, description of "
                 "the image will be reset to empty",
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing
        image_client = self.app.client_manager.image

        image_id = osc_utils.find_resource(
            image_client.images, parsed_args.image).id

        data = client.images.update_image(
            image_id, user_name=parsed_args.username,
            desc=parsed_args.description).image

        data['tags'] = osc_utils.format_list(data['tags'])

        data = utils.prepare_data(data, IMAGE_FIELDS)

        return self.dict2columns(data)


class UnregisterImage(command.Command):
    """Unregister image(s)"""

    log = logging.getLogger(__name__ + ".RegisterImage")

    def get_parser(self, prog_name):
        parser = super(UnregisterImage, self).get_parser(prog_name)
        parser.add_argument(
            "image",
            metavar="<image>",
            nargs="+",
            help="Name(s) or id(s) of the image(s) to unregister",
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing
        for image in parsed_args.image:
            image_id = utils.get_resource_id(client.images, image)
            client.images.unregister_image(image_id)
            sys.stdout.write(
                'Image "{image}" has been unregistered '
                'successfully.\n'.format(image=image))


class SetImageTags(show.ShowOne):
    """Set image tags (Replace current image tags with provided ones)"""

    log = logging.getLogger(__name__ + ".AddImageTags")

    def get_parser(self, prog_name):
        parser = super(SetImageTags, self).get_parser(prog_name)
        parser.add_argument(
            "image",
            metavar="<image>",
            help="Name or id of the image",
        )
        parser.add_argument(
            '--tags',
            metavar="<tag>",
            nargs="+",
            required=True,
            help="Tag(s) to set [REQUIRED]"
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing

        image_id = utils.get_resource_id(client.images, parsed_args.image)
        data = client.images.update_tags(image_id, parsed_args.tags).to_dict()

        data['tags'] = osc_utils.format_list(data['tags'])

        data = utils.prepare_data(data, IMAGE_FIELDS)

        return self.dict2columns(data)


class AddImageTags(show.ShowOne):
    """Add image tags"""

    log = logging.getLogger(__name__ + ".AddImageTags")

    def get_parser(self, prog_name):
        parser = super(AddImageTags, self).get_parser(prog_name)
        parser.add_argument(
            "image",
            metavar="<image>",
            help="Name or id of the image",
        )
        parser.add_argument(
            '--tags',
            metavar="<tag>",
            nargs="+",
            required=True,
            help="Tag(s) to add [REQUIRED]"
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing

        image = utils.get_resource(client.images, parsed_args.image)
        parsed_args.tags.extend(image.tags)
        data = client.images.update_tags(
            image.id, list(set(parsed_args.tags))).to_dict()

        data['tags'] = osc_utils.format_list(data['tags'])

        data = utils.prepare_data(data, IMAGE_FIELDS)

        return self.dict2columns(data)


class RemoveImageTags(show.ShowOne):
    """Remove image tags"""

    log = logging.getLogger(__name__ + ".RemoveImageTags")

    def get_parser(self, prog_name):
        parser = super(RemoveImageTags, self).get_parser(prog_name)
        parser.add_argument(
            "image",
            metavar="<image>",
            help="Name or id of the image",
        )
        group = parser.add_mutually_exclusive_group()
        group.add_argument(
            '--tags',
            metavar="<tag>",
            nargs="+",
            help="Tag(s) to remove"
        ),
        group.add_argument(
            '--all',
            action='store_true',
            default=False,
            help='Remove all tags from image',
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing

        image = utils.get_resource(client.images, parsed_args.image)

        if parsed_args.all:
            data = client.images.update_tags(image.id, []).to_dict()
        else:
            parsed_args.tags = parsed_args.tags or []
            new_tags = list(set(image.tags) - set(parsed_args.tags))
            data = client.images.update_tags(image.id, new_tags).to_dict()

        data['tags'] = osc_utils.format_list(data['tags'])

        data = utils.prepare_data(data, IMAGE_FIELDS)

        return self.dict2columns(data)



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from saharaclient.api import job_binaries as jb
from saharaclient.tests.unit import base

import json


class JobBinaryTest(base.BaseTestCase):
    body = {
        'name': 'name',
        'url': 'url',
        'description': 'descr',
        'extra': {
            'user': 'user',
            'password': '123'
        }
    }

    update_body = {
        'name': 'Updatedname',
        'url': 'Updatedurl',
        'description': 'Updateddescr',
        'extra': {
            'user': 'user',
            'password': 'Updated123'
        }
    }

    def test_create_job_binary(self):
        url = self.URL + '/job-binaries'
        self.responses.post(url, status_code=202,
                            json={'job_binary': self.body})

        resp = self.client.job_binaries.create(**self.body)

        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual(self.body,
                         json.loads(self.responses.last_request.body))
        self.assertIsInstance(resp, jb.JobBinaries)
        self.assertFields(self.body, resp)

    def test_job_binary_list(self):
        url = self.URL + '/job-binaries'
        self.responses.get(url, json={'binaries': [self.body]})

        resp = self.client.job_binaries.list()

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp[0], jb.JobBinaries)
        self.assertFields(self.body, resp[0])

    def test_job_binary_get(self):
        url = self.URL + '/job-binaries/id'
        self.responses.get(url, json={'job_binary': self.body})

        resp = self.client.job_binaries.get('id')

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp, jb.JobBinaries)
        self.assertFields(self.body, resp)

    def test_job_binary_delete(self):
        url = self.URL + '/job-binaries/id'
        self.responses.delete(url, status_code=204)

        self.client.job_binaries.delete('id')

        self.assertEqual(url, self.responses.last_request.url)

    def test_job_binary_get_file(self):
        url = self.URL + '/job-binaries/id/data'
        self.responses.get(url, text='data')

        resp = self.client.job_binaries.get_file('id')

        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual(b'data', resp)

    def test_job_binary_update(self):
        url = self.URL + '/job-binaries/id'
        self.responses.put(url,
                           status_code=202,
                           json={'job_binary': self.update_body})

        resp = self.client.job_binaries.update("id", self.update_body)
        self.assertEqual(self.update_body["name"], resp.name)



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from saharaclient.api import cluster_templates as ct
from saharaclient.tests.unit import base

import json


class ClusterTemplateTest(base.BaseTestCase):
    body = {
        'name': 'name',
        'description': 'description',
        'plugin_name': 'plugin',
        'hadoop_version': '1',
        'node_groups': {
            'name': 'master-node',
            'flavor_id': '2',
            'node_processes': ['namenode'],
            'count': 1
        },
        "use_autoconfig": False,
    }

    update_json = {
        "cluster_template": {
            'name': 'UpdatedName',
            'description': 'Updated description',
            'plugin_name': 'plugin',
            'hadoop_version': '1',
            'node_groups': {
                'name': 'master-node',
                'flavor_id': '3',
                'node_processes': ['namenode', 'datanode'],
                'count': 1
            },
            "use_autoconfig": True,
        }
    }

    def test_create_cluster_template(self):
        url = self.URL + '/cluster-templates'
        self.responses.post(url, status_code=202,
                            json={'cluster_template': self.body})

        resp = self.client.cluster_templates.create(**self.body)

        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual(self.body,
                         json.loads(self.responses.last_request.body))
        self.assertIsInstance(resp, ct.ClusterTemplate)
        self.assertFields(self.body, resp)

    def test_cluster_template_list(self):
        url = self.URL + '/cluster-templates'
        self.responses.get(url, json={'cluster_templates': [self.body]})

        resp = self.client.cluster_templates.list()

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp[0], ct.ClusterTemplate)
        self.assertFields(self.body, resp[0])

    def test_cluster_template_get(self):
        url = self.URL + '/cluster-templates/id'
        self.responses.get(url, json={'cluster_template': self.body})

        resp = self.client.cluster_templates.get('id')

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp, ct.ClusterTemplate)
        self.assertFields(self.body, resp)

    def test_cluster_template_delete(self):
        url = self.URL + '/cluster-templates/id'
        self.responses.delete(url, status_code=204)

        self.client.cluster_templates.delete('id')

        self.assertEqual(url, self.responses.last_request.url)

    def test_cluster_template_update(self):
        url = self.URL + '/cluster-templates'
        self.responses.post(url, status_code=202,
                            json={'cluster_template': self.body})
        resp = self.client.cluster_templates.create(**self.body)

        update_url = self.URL + '/cluster-templates/id'
        self.responses.put(update_url, status_code=202, json=self.update_json)

        # check that all parameters will be updated
        updated = self.client.cluster_templates.update(
            "id",
            resp.name,
            resp.plugin_name,
            resp.hadoop_version,
            description=getattr(resp, "description", None),
            cluster_configs=getattr(resp, "cluster_configs", None),
            node_groups=getattr(resp, "node_groups", None),
            anti_affinity=getattr(resp, "anti_affinity", None),
            net_id=getattr(resp, "neutron_management_network", None),
            default_image_id=getattr(resp, "default_image_id", None),
            use_autoconfig=True,
        )

        self.assertIsInstance(updated, ct.ClusterTemplate)
        self.assertFields(self.update_json["cluster_template"], updated)

        # check that parameters will not be updated
        self.client.cluster_templates.update("id")
        self.assertEqual(update_url, self.responses.last_request.url)
        self.assertEqual({},
                         json.loads(self.responses.last_request.body))

        # check that all parameters will be unset
        unset_json = {
            'anti_affinity': None, 'cluster_configs': None,
            'default_image_id': None, 'description': None,
            'hadoop_version': None, 'is_protected': None, 'is_public': None,
            'name': None, 'net_id': None,
            'node_groups': None, 'plugin_name': None, 'shares': None,
            'use_autoconfig': None}

        req_json = unset_json.copy()
        req_json['neutron_management_network'] = req_json.pop('net_id')

        self.client.cluster_templates.update("id", **unset_json)
        self.assertEqual(update_url, self.responses.last_request.url)
        self.assertEqual(req_json,
                         json.loads(self.responses.last_request.body))



# Copyright (c) 2014 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

import testtools

from saharaclient.tests.integration.configs import config as cfg
import saharaclient.tests.integration.tests.full_test_driver as driver

cfg = cfg.ITConfig()
vanilla = cfg.vanilla_config


class ClusterVanilla(driver.FullTestDriver):

    @testtools.skipIf(vanilla.SKIP_ALL_TESTS_FOR_PLUGIN,
                      'All tests for Vanilla plugin were skipped')
    def test_cluster_vanilla(self):
        marker = str(os.getpid())
        ng_templates = [
            {
                "values": {
                    "name": "w-%s" % marker,
                    "flavor_id": "2",
                    "plugin_name": vanilla.PLUGIN_NAME,
                    "hadoop_version": vanilla.HADOOP_VERSION,
                    "node_processes": ["tasktracker", "datanode"],
                    "floating_ip_pool": self.floating_ip_pool
                },
                "count": 1
            },
            {
                "values": {
                    "name": "m-%s" % marker,
                    "flavor_id": "2",
                    "plugin_name": vanilla.PLUGIN_NAME,
                    "hadoop_version": vanilla.HADOOP_VERSION,
                    "node_processes": ["jobtracker", "namenode",
                                       "oozie"],
                    "floating_ip_pool": self.floating_ip_pool
                },
                "count": 1
            }
        ]
        self.drive_full_test(vanilla, ng_templates)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from os import path
import sys

from cliff import command
from cliff import lister
from cliff import show
from openstackclient.common import exceptions
from openstackclient.common import utils as osc_utils
from oslo_log import log as logging
from oslo_serialization import jsonutils

from saharaclient.api import base
from saharaclient.osc.v1 import utils

JOB_BINARY_FIELDS = ['name', 'id', 'url', 'description', 'is_public',
                     'is_protected']


class CreateJobBinary(show.ShowOne):
    """Creates job binary"""

    log = logging.getLogger(__name__ + ".CreateJobBinary")

    def get_parser(self, prog_name):
        parser = super(CreateJobBinary, self).get_parser(prog_name)

        parser.add_argument(
            '--name',
            metavar="<name>",
            help="Name of the job binary [REQUIRED if JSON is not provided]",
        )
        creation_type = parser.add_mutually_exclusive_group()
        creation_type.add_argument(
            '--data',
            metavar='<file>',
            help='File that will be stored in the internal DB [REQUIRED if '
                 'JSON and URL are not provided]'
        )
        creation_type.add_argument(
            '--url',
            metavar='<url>',
            help='URL for the job binary [REQUIRED if JSON and file are '
                 'not provided]'
        )
        parser.add_argument(
            '--description',
            metavar="<description>",
            help="Description of the job binary"
        )
        parser.add_argument(
            '--username',
            metavar='<username>',
            help='Username for accessing the job binary URL',
        )
        password = parser.add_mutually_exclusive_group()
        password.add_argument(
            '--password',
            metavar='<password>',
            help='Password for accessing the job binary URL',
        )
        password.add_argument(
            '--password-prompt',
            dest="password_prompt",
            action="store_true",
            help='Prompt interactively for password',
        )
        parser.add_argument(
            '--public',
            action='store_true',
            default=False,
            help='Make the job binary public',
        )
        parser.add_argument(
            '--protected',
            action='store_true',
            default=False,
            help='Make the job binary protected',
        )
        parser.add_argument(
            '--json',
            metavar='<filename>',
            help='JSON representation of the job binary. Other '
                 'arguments will not be taken into account if this one is '
                 'provided'
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing

        if parsed_args.json:
            blob = osc_utils.read_blob_file_contents(parsed_args.json)
            try:
                template = jsonutils.loads(blob)
            except ValueError as e:
                raise exceptions.CommandError(
                    'An error occurred when reading '
                    'template from file %s: %s' % (parsed_args.json, e))
            data = client.job_binaries.create(**template).to_dict()
        else:
            if parsed_args.data:
                data = open(parsed_args.data).read()
                jbi_id = client.job_binary_internals.create(
                    parsed_args.name, data).id
                parsed_args.url = 'internal-db://' + jbi_id

            if parsed_args.password_prompt:
                parsed_args.password = osc_utils.get_password(
                    self.app.stdin, confirm=False)

            if parsed_args.password and not parsed_args.username:
                raise exceptions.CommandError(
                    'Username via --username should be provided with password')

            if parsed_args.username and not parsed_args.password:
                raise exceptions.CommandError(
                    'Password should be provided via --password or entered '
                    'interactively with --password-prompt')

            if parsed_args.password and parsed_args.username:
                extra = {
                    'user': parsed_args.username,
                    'password': parsed_args.password
                }
            else:
                extra = None

            data = client.job_binaries.create(
                name=parsed_args.name, url=parsed_args.url,
                description=parsed_args.description, extra=extra,
                is_public=parsed_args.public,
                is_protected=parsed_args.protected).to_dict()

        data = utils.prepare_data(data, JOB_BINARY_FIELDS)

        return self.dict2columns(data)


class ListJobBinaries(lister.Lister):
    """Lists job binaries"""

    log = logging.getLogger(__name__ + ".ListJobBinaries")

    def get_parser(self, prog_name):
        parser = super(ListJobBinaries, self).get_parser(prog_name)
        parser.add_argument(
            '--long',
            action='store_true',
            default=False,
            help='List additional fields in output',
        )
        parser.add_argument(
            '--name',
            metavar="<name-substring>",
            help="List job binaries with specific substring in the "
                 "name"
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing

        data = client.job_binaries.list()

        if parsed_args.name:
            data = utils.get_by_name_substring(data, parsed_args.name)

        if parsed_args.long:
            columns = ('name', 'id', 'url', 'description', 'is_public',
                       'is_protected')
            column_headers = utils.prepare_column_headers(columns)

        else:
            columns = ('name', 'id', 'url')
            column_headers = utils.prepare_column_headers(columns)

        return (
            column_headers,
            (osc_utils.get_item_properties(
                s,
                columns
            ) for s in data)
        )


class ShowJobBinary(show.ShowOne):
    """Display job binary details"""

    log = logging.getLogger(__name__ + ".ShowJobBinary")

    def get_parser(self, prog_name):
        parser = super(ShowJobBinary, self).get_parser(prog_name)
        parser.add_argument(
            "job_binary",
            metavar="<job-binary>",
            help="Name or ID of the job binary to display",
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing

        data = utils.get_resource(
            client.job_binaries, parsed_args.job_binary).to_dict()

        data = utils.prepare_data(data, JOB_BINARY_FIELDS)

        return self.dict2columns(data)


class DeleteJobBinary(command.Command):
    """Deletes job binary"""

    log = logging.getLogger(__name__ + ".DeleteJobBinary")

    def get_parser(self, prog_name):
        parser = super(DeleteJobBinary, self).get_parser(prog_name)
        parser.add_argument(
            "job_binary",
            metavar="<job-binary>",
            nargs="+",
            help="Name(s) or id(s) of the job binary(ies) to delete",
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing
        for jb in parsed_args.job_binary:
            jb = utils.get_resource(client.job_binaries, jb)
            if jb.url.startswith("internal-db"):
                jbi_id = jb.url.replace('internal-db://', '')
                try:
                    client.job_binary_internals.delete(jbi_id)
                except base.APIException as ex:
                    # check if job binary internal was already deleted for
                    # some reasons
                    if not ex.error_code == '404':
                        raise
            client.job_binaries.delete(jb.id)
            sys.stdout.write(
                'Job binary "{jb}" has been removed '
                'successfully.\n'.format(jb=jb))


class UpdateJobBinary(show.ShowOne):
    """Updates job binary"""

    log = logging.getLogger(__name__ + ".UpdateJobBinary")

    def get_parser(self, prog_name):
        parser = super(UpdateJobBinary, self).get_parser(prog_name)

        parser.add_argument(
            'job_binary',
            metavar="<job-binary>",
            help="Name or ID of the job binary",
        )
        parser.add_argument(
            '--name',
            metavar="<name>",
            help="New name of the job binary",
        )
        parser.add_argument(
            '--url',
            metavar='<url>',
            help='URL for the job binary [Internal DB URL can not be updated]'
        )
        parser.add_argument(
            '--description',
            metavar="<description>",
            help='Description of the job binary'
        )
        parser.add_argument(
            '--username',
            metavar='<username>',
            help='Username for accessing the job binary URL',
        )
        password = parser.add_mutually_exclusive_group()
        password.add_argument(
            '--password',
            metavar='<password>',
            help='Password for accessing the job binary URL',
        )
        password.add_argument(
            '--password-prompt',
            dest="password_prompt",
            action="store_true",
            help='Prompt interactively for password',
        )
        public = parser.add_mutually_exclusive_group()
        public.add_argument(
            '--public',
            action='store_true',
            help='Make the job binary public (Visible from other tenants)',
            dest='is_public'
        )
        public.add_argument(
            '--private',
            action='store_false',
            help='Make the job binary private (Visible only from this tenant)',
            dest='is_public'
        )
        protected = parser.add_mutually_exclusive_group()
        protected.add_argument(
            '--protected',
            action='store_true',
            help='Make the job binary protected',
            dest='is_protected'
        )
        protected.add_argument(
            '--unprotected',
            action='store_false',
            help='Make the job binary unprotected',
            dest='is_protected'
        )
        parser.add_argument(
            '--json',
            metavar='<filename>',
            help='JSON representation of the update object. Other '
                 'arguments will not be taken into account if this one is '
                 'provided'
        )
        parser.set_defaults(is_public=None, is_protected=None)

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing

        jb_id = utils.get_resource_id(
            client.job_binaries, parsed_args.job_binary)

        if parsed_args.json:
            blob = osc_utils.read_blob_file_contents(parsed_args.json)
            try:
                template = jsonutils.loads(blob)
            except ValueError as e:
                raise exceptions.CommandError(
                    'An error occurred when reading '
                    'template from file %s: %s' % (parsed_args.json, e))
            data = client.job_binaries.update(jb_id, template).to_dict()
        else:
            if parsed_args.password_prompt:
                parsed_args.password = osc_utils.get_password(
                    self.app.stdin, confirm=False)

            extra = {}
            if parsed_args.password:
                extra['password'] = parsed_args.password
            if parsed_args.username:
                extra['user'] = parsed_args.username
            if not extra:
                extra = None

            update_fields = utils.create_dict_from_kwargs(
                name=parsed_args.name, url=parsed_args.url,
                description=parsed_args.description,
                extra=extra, is_public=parsed_args.is_public,
                is_protected=parsed_args.is_protected
            )

            data = client.job_binaries.update(
                jb_id, update_fields).to_dict()

        data = utils.prepare_data(data, JOB_BINARY_FIELDS)

        return self.dict2columns(data)


class DownloadJobBinary(command.Command):
    """Downloads job binary"""

    log = logging.getLogger(__name__ + ".DownloadJobBinary")

    def get_parser(self, prog_name):
        parser = super(DownloadJobBinary, self).get_parser(prog_name)
        parser.add_argument(
            "job_binary",
            metavar="<job-binary>",
            help="Name or ID of the job binary to download",
        )
        parser.add_argument(
            '--file',
            metavar="<file>",
            help='Destination file (defaults to job binary name)',
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing

        if not parsed_args.file:
            parsed_args.file = parsed_args.job_binary

        jb_id = utils.get_resource_id(
            client.job_binaries, parsed_args.job_binary)
        data = client.job_binaries.get_file(jb_id)

        if path.exists(parsed_args.file):
            self.log.error('File "%s" already exists. Chose another one with '
                           '--file argument.' % parsed_args.file)
        else:
            with open(parsed_args.file, 'w') as f:
                f.write(data)
            sys.stdout.write(
                'Job binary "{jb}" has been downloaded '
                'successfully.\n'.format(jb=parsed_args.job_binary))



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from saharaclient.api import clusters as cl
from saharaclient.tests.unit import base

import json


class ClusterTest(base.BaseTestCase):
    body = {
        'name': 'name',
        'plugin_name': 'fake',
        'hadoop_version': '0.1',
        'cluster_template_id': 'id',
    }

    body_with_count = {
        'name': 'name',
        'plugin_name': 'fake',
        'hadoop_version': '0.1',
        'cluster_template_id': 'id',
        'count': 2
    }

    body_with_progress = {
        'name': 'name',
        'plugin_name': 'fake',
        'hadoop_version': '0.1',
        'cluster_template_id': 'id',
        "provision_progress": []
    }

    test_shares = [
        {
            "id": "bd71d2d5-60a0-4ed9-a3d2-ad312c368880",
            "path": "/mnt/manila",
            "access_level": "rw"
        }
    ]

    def test_create_cluster_with_template(self,):
        url = self.URL + '/clusters'
        self.responses.post(url, status_code=202, json={'cluster': self.body})

        resp = self.client.clusters.create(**self.body)

        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual(self.body,
                         json.loads(self.responses.last_request.body))
        self.assertIsInstance(resp, cl.Cluster)
        self.assertFields(self.body, resp)

    def test_create_cluster_without_template(self):
        body = self.body.copy()
        del body['cluster_template_id']
        body.update({'default_image_id': 'image_id', 'cluster_configs': {},
                     'node_groups': ['ng1', 'ng2']})

        url = self.URL + '/clusters'
        self.responses.post(url, status_code=202, json={'cluster': body})

        resp = self.client.clusters.create(**body)

        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual(body, json.loads(self.responses.last_request.body))
        self.assertIsInstance(resp, cl.Cluster)
        self.assertFields(body, resp)

    def test_create_multiple_clusters(self):
        url = self.URL + '/clusters/multiple'
        self.responses.post(url, status_code=202,
                            json={'clusters': ['id1', 'id2']})

        resp = self.client.clusters.create(**self.body_with_count)

        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual(self.body_with_count,
                         json.loads(self.responses.last_request.body))
        self.assertIsInstance(resp, cl.Cluster)
        self.assertFields({'clusters': ['id1', 'id2']}, resp)

    def test_clusters_list(self):
        url = self.URL + '/clusters'
        self.responses.get(url, json={'clusters': [self.body]})

        resp = self.client.clusters.list()

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp[0], cl.Cluster)
        self.assertFields(self.body, resp[0])

    def test_clusters_get(self):
        url = self.URL + '/clusters/id?show_progress=False'
        self.responses.get(url, json={'cluster': self.body})

        resp = self.client.clusters.get('id')

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp, cl.Cluster)
        self.assertFields(self.body, resp)

    def test_clusters_get_with_progress(self):
        url = self.URL + '/clusters/id?show_progress=True'
        self.responses.get(url, json={'cluster': self.body_with_progress})

        resp = self.client.clusters.get('id', show_progress=True)

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp, cl.Cluster)
        self.assertFields(self.body, resp)

    def test_clusters_scale(self):
        url = self.URL + '/clusters/id'
        self.responses.put(url, status_code=202, json=self.body)

        scale_body = {
            'resize_node_groups': [
                {
                    'count': 2,
                    'name': 'name1'
                },
            ],
            'add_node_groups': [
                {
                    'count': 1,
                    'name': 'name2',
                    'node_group_template_id': 'id'
                }
            ]
        }

        resp = self.client.clusters.scale('id', scale_body)

        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual(scale_body,
                         json.loads(self.responses.last_request.body))
        self.assertIsInstance(resp, cl.Cluster)
        self.assertFields(self.body, resp)

    def test_clusters_delete(self):
        url = self.URL + '/clusters/id'
        self.responses.delete(url, status_code=204)

        self.client.clusters.delete('id')

        self.assertEqual(url, self.responses.last_request.url)

    def test_clusters_update(self):
        url = self.URL + '/clusters/id'

        update_body = {
            'name': 'new_name',
            'description': 'descr'
        }

        self.responses.patch(url, status_code=202, json=update_body)

        # check that all parameters will be updated
        resp = self.client.clusters.update('id', name='new_name',
                                           description='descr')

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp, cl.Cluster)
        self.assertEqual(update_body,
                         json.loads(self.responses.last_request.body))

        # check that parameters will not be updated
        self.client.clusters.update("id")
        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual({},
                         json.loads(self.responses.last_request.body))

        # check that all parameters will be unset
        unset_json = {
            "name": None, "description": None, "is_public": None,
            "is_protected": None, "shares": None
        }

        self.client.clusters.update("id", **unset_json)
        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual(unset_json,
                         json.loads(self.responses.last_request.body))

    def test_clusters_update_share(self):
        url = self.URL + '/clusters/id'

        update_body = {
            'name': 'new_name',
            'description': 'descr',
            'shares': self.test_shares
        }

        self.responses.patch(url, status_code=202, json=update_body)

        resp = self.client.clusters.update('id', name='new_name',
                                           description='descr',
                                           shares=self.test_shares)

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp, cl.Cluster)
        self.assertEqual(update_body,
                         json.loads(self.responses.last_request.body))



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from os import path
import sys

from cliff import command
from cliff import lister
from cliff import show
from openstackclient.common import utils as osc_utils
from oslo_log import log as logging
from oslo_serialization import jsonutils

from saharaclient.osc.v1 import utils


class ListPlugins(lister.Lister):
    """Lists plugins"""

    log = logging.getLogger(__name__ + ".ListPlugins")

    def get_parser(self, prog_name):
        parser = super(ListPlugins, self).get_parser(prog_name)
        parser.add_argument(
            '--long',
            action='store_true',
            default=False,
            help='List additional fields in output',
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing
        data = client.plugins.list()

        if parsed_args.long:
            columns = ('name', 'title', 'versions', 'description')
            column_headers = utils.prepare_column_headers(columns)

        else:
            columns = ('name', 'versions')
            column_headers = utils.prepare_column_headers(columns)

        return (
            column_headers,
            (osc_utils.get_item_properties(
                s,
                columns,
                formatters={
                    'versions': osc_utils.format_list
                },
            ) for s in data)
        )


class ShowPlugin(show.ShowOne):
    """Display plugin details"""

    log = logging.getLogger(__name__ + ".ShowPlugin")

    def get_parser(self, prog_name):
        parser = super(ShowPlugin, self).get_parser(prog_name)
        parser.add_argument(
            "plugin",
            metavar="<plugin>",
            help="Name of the plugin to display",
        )
        parser.add_argument(
            "--version",
            help='Version of the plugin to display'
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing

        if parsed_args.version:
            data = client.plugins.get_version_details(
                parsed_args.plugin, parsed_args.version).to_dict()

            processes = data.pop('node_processes')
            for k, v in processes.items():
                processes[k] = osc_utils.format_list(v)
            data['required_image_tags'] = osc_utils.format_list(
                data['required_image_tags'])

            data = utils.prepare_data(
                data, ['required_image_tags', 'name', 'description', 'title'])

            data = zip(*sorted(data.items()) + [('', ''), (
                'Service:', 'Available processes:'), ('', '')] + sorted(
                processes.items()))
        else:
            data = client.plugins.get(parsed_args.plugin).to_dict()
            data['versions'] = osc_utils.format_list(data['versions'])
            data = utils.prepare_data(
                data, ['versions', 'name', 'description', 'title'])
            data = self.dict2columns(data)

        return data


class GetPluginConfigs(command.Command):
    """Get plugin configs"""

    log = logging.getLogger(__name__ + ".GetPluginConfigs")

    def get_parser(self, prog_name):
        parser = super(GetPluginConfigs, self).get_parser(prog_name)
        parser.add_argument(
            "plugin",
            metavar="<plugin>",
            help="Name of the plugin to provide config information about",
        )
        parser.add_argument(
            "version",
            metavar="<version>",
            help="Version of the plugin to provide config information about",
        )
        parser.add_argument(
            '--file',
            metavar="<file>",
            help='Destination file (defaults to plugin name)',
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing

        if not parsed_args.file:
            parsed_args.file = parsed_args.plugin

        data = client.plugins.get_version_details(
            parsed_args.plugin, parsed_args.version).to_dict()

        if path.exists(parsed_args.file):
            self.log.error('File "%s" already exists. Chose another one with '
                           '--file argument.' % parsed_args.file)
        else:
            with open(parsed_args.file, 'w') as f:
                jsonutils.dump(data, f, indent=4)
            sys.stdout.write(
                '"%(plugin)s" plugin configs was saved in "%(file)s"'
                'file' % {'plugin': parsed_args.plugin,
                          'file': parsed_args.file})



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from openstackclient.tests import utils as osc_utils

from saharaclient.api import cluster_templates as api_ct
from saharaclient.api import node_group_templates as api_ngt
from saharaclient.osc.v1 import cluster_templates as osc_ct
from saharaclient.tests.unit.osc.v1 import fakes

CT_INFO = {
    "description": "Cluster template for tests",
    "use_autoconfig": True,
    "is_default": False,
    "node_groups": [
        {
            "count": 2,
            "id": "d29631fc-0fad-434b-80aa-7a3e9526f57c",
            "name": "fakeng",
            "plugin_name": 'fake',
            "hadoop_version": '0.1'
        }
    ],
    "hadoop_version": "0.1",
    "is_public": False,
    "plugin_name": "fake",
    "id": "0647061f-ab98-4c89-84e0-30738ea55750",
    "anti_affinity": [],
    "name": "template",
    "is_protected": False
}


class TestClusterTemplates(fakes.TestDataProcessing):
    def setUp(self):
        super(TestClusterTemplates, self).setUp()
        self.ct_mock = (
            self.app.client_manager.data_processing.cluster_templates)
        self.ngt_mock = (
            self.app.client_manager.data_processing.node_group_templates)
        self.ct_mock.reset_mock()
        self.ngt_mock.reset_mock()


class TestCreateClusterTemplate(TestClusterTemplates):
    # TODO(apavlov): check for creation with --json
    def setUp(self):
        super(TestCreateClusterTemplate, self).setUp()
        self.ct_mock.create.return_value = api_ct.ClusterTemplate(
            None, CT_INFO)
        self.ngt_mock.find_unique.return_value = api_ngt.NodeGroupTemplate(
            None, CT_INFO['node_groups'][0])

        # Command to test
        self.cmd = osc_ct.CreateClusterTemplate(self.app, None)

    def test_ct_create_minimum_options(self):
        arglist = ['--name', 'template', '--node-groups', 'fakeng:2']
        verifylist = [('name', 'template'),
                      ('node_groups', ['fakeng:2'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.ct_mock.create.assert_called_once_with(
            description=None, hadoop_version='0.1', is_protected=False,
            is_public=False, name='template', node_groups=[
                {'count': 2, 'name': 'fakeng',
                 'node_group_template_id':
                     'd29631fc-0fad-434b-80aa-7a3e9526f57c'}],
            plugin_name='fake', use_autoconfig=False, shares=None,
            cluster_configs=None)

    def test_ct_create_all_options(self):
        arglist = ['--name', 'template', '--node-groups', 'fakeng:2',
                   '--anti-affinity', 'datanode',
                   '--description', 'descr',
                   '--autoconfig', '--public', '--protected']

        verifylist = [('name', 'template'),
                      ('node_groups', ['fakeng:2']),
                      ('description', 'descr'), ('autoconfig', True),
                      ('public', True), ('protected', True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.ct_mock.create.assert_called_once_with(
            description='descr', hadoop_version='0.1', is_protected=True,
            is_public=True, name='template', node_groups=[
                {'count': 2, 'name': 'fakeng',
                 'node_group_template_id':
                     'd29631fc-0fad-434b-80aa-7a3e9526f57c'}],
            plugin_name='fake', use_autoconfig=True, shares=None,
            cluster_configs=None)

        # Check that columns are correct
        expected_columns = ('Anti affinity', 'Description', 'Id', 'Is default',
                            'Is protected', 'Is public', 'Name', 'Node groups',
                            'Plugin name', 'Use autoconfig', 'Version')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = ('', 'Cluster template for tests',
                         '0647061f-ab98-4c89-84e0-30738ea55750', False, False,
                         False, 'template', 'fakeng:2', 'fake', True, '0.1')
        self.assertEqual(expected_data, data)


class TestListClusterTemplates(TestClusterTemplates):
    def setUp(self):
        super(TestListClusterTemplates, self).setUp()
        self.ct_mock.list.return_value = [api_ct.ClusterTemplate(
            None, CT_INFO)]

        # Command to test
        self.cmd = osc_ct.ListClusterTemplates(self.app, None)

    def test_ct_list_no_options(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Name', 'Id', 'Plugin name', 'Version']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [('template', '0647061f-ab98-4c89-84e0-30738ea55750',
                          'fake', '0.1')]
        self.assertEqual(expected_data, list(data))

    def test_ct_list_long(self):
        arglist = ['--long']
        verifylist = [('long', True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Name', 'Id', 'Plugin name', 'Version',
                            'Node groups', 'Description']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [('template', '0647061f-ab98-4c89-84e0-30738ea55750',
                          'fake', '0.1', 'fakeng:2',
                          'Cluster template for tests')]
        self.assertEqual(expected_data, list(data))

    def test_ct_list_extra_search_opts(self):
        arglist = ['--plugin', 'fake', '--version', '0.1', '--name', 'templ']
        verifylist = [('plugin', 'fake'), ('version', '0.1'),
                      ('name', 'templ')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Name', 'Id', 'Plugin name', 'Version']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [('template', '0647061f-ab98-4c89-84e0-30738ea55750',
                          'fake', '0.1')]
        self.assertEqual(expected_data, list(data))


class TestShowClusterTemplate(TestClusterTemplates):
    def setUp(self):
        super(TestShowClusterTemplate, self).setUp()
        self.ct_mock.find_unique.return_value = api_ct.ClusterTemplate(
            None, CT_INFO)

        # Command to test
        self.cmd = osc_ct.ShowClusterTemplate(self.app, None)

    def test_ct_show(self):
        arglist = ['template']
        verifylist = [('cluster_template', 'template')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.ct_mock.find_unique.assert_called_once_with(name='template')

        # Check that columns are correct
        expected_columns = ('Anti affinity', 'Description', 'Id', 'Is default',
                            'Is protected', 'Is public', 'Name', 'Node groups',
                            'Plugin name', 'Use autoconfig', 'Version')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = (
            '', 'Cluster template for tests',
            '0647061f-ab98-4c89-84e0-30738ea55750', False, False, False,
            'template', 'fakeng:2', 'fake', True, '0.1')
        self.assertEqual(expected_data, data)


class TestDeleteClusterTemplate(TestClusterTemplates):
    def setUp(self):
        super(TestDeleteClusterTemplate, self).setUp()
        self.ct_mock.find_unique.return_value = api_ct.ClusterTemplate(
            None, CT_INFO)

        # Command to test
        self.cmd = osc_ct.DeleteClusterTemplate(self.app, None)

    def test_ct_delete(self):
        arglist = ['template']
        verifylist = [('cluster_template', ['template'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.ct_mock.delete.assert_called_once_with(
            '0647061f-ab98-4c89-84e0-30738ea55750')


class TestUpdateClusterTemplate(TestClusterTemplates):
    # TODO(apavlov): check for update with --json
    def setUp(self):
        super(TestUpdateClusterTemplate, self).setUp()
        self.ct_mock.update.return_value = api_ct.ClusterTemplate(
            None, CT_INFO)
        self.ct_mock.find_unique.return_value = api_ct.ClusterTemplate(
            None, CT_INFO)
        self.ngt_mock.find_unique.return_value = api_ngt.NodeGroupTemplate(
            None, CT_INFO['node_groups'][0])

        # Command to test
        self.cmd = osc_ct.UpdateClusterTemplate(self.app, None)

    def test_ct_update_no_options(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException, self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_ct_update_nothing_updated(self):
        arglist = ['template']
        verifylist = [('cluster_template', 'template')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        self.ct_mock.update.assert_called_once_with(
            '0647061f-ab98-4c89-84e0-30738ea55750')

    def test_ct_update_all_options(self):
        arglist = ['template', '--name', 'template', '--node-groups',
                   'fakeng:2', '--anti-affinity', 'datanode',
                   '--description', 'descr', '--autoconfig-enable',
                   '--public', '--protected']

        verifylist = [('cluster_template', 'template'), ('name', 'template'),
                      ('node_groups', ['fakeng:2']),
                      ('description', 'descr'), ('use_autoconfig', True),
                      ('is_public', True), ('is_protected', True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.ct_mock.update.assert_called_once_with(
            '0647061f-ab98-4c89-84e0-30738ea55750', description='descr',
            hadoop_version='0.1', is_protected=True, is_public=True,
            name='template',
            node_groups=[
                {'count': 2, 'name': 'fakeng',
                 'node_group_template_id':
                     'd29631fc-0fad-434b-80aa-7a3e9526f57c'}],
            plugin_name='fake', use_autoconfig=True)

        # Check that columns are correct
        expected_columns = ('Anti affinity', 'Description', 'Id', 'Is default',
                            'Is protected', 'Is public', 'Name', 'Node groups',
                            'Plugin name', 'Use autoconfig', 'Version')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = ('', 'Cluster template for tests',
                         '0647061f-ab98-4c89-84e0-30738ea55750', False, False,
                         False, 'template', 'fakeng:2', 'fake', True, '0.1')
        self.assertEqual(expected_data, data)

    def test_ct_update_private_unprotected(self):
        arglist = ['template', '--private', '--unprotected']
        verifylist = [('cluster_template', 'template'),
                      ('is_protected', False), ('is_public', False)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        self.ct_mock.update.assert_called_once_with(
            '0647061f-ab98-4c89-84e0-30738ea55750', is_protected=False,
            is_public=False)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import sys

from cliff import command
from cliff import lister
from cliff import show
from openstackclient.common import exceptions
from openstackclient.common import utils as osc_utils
from oslo_log import log as logging

from saharaclient.osc.v1 import utils

CLUSTER_FIELDS = ["cluster_template_id", "use_autoconfig", "user_keypair_id",
                  "status", "image", "node_groups", "id",
                  "anti_affinity", "version", "name", "is_transient",
                  "is_protected", "description", "is_public",
                  "neutron_management_network", "plugin_name"]


def _format_node_groups_list(node_groups):
    return ', '.join(
        ['%s:%s' % (ng['name'], ng['count']) for ng in node_groups])


def _format_cluster_output(data):
    data['version'] = data.pop('hadoop_version')
    data['image'] = data.pop('default_image_id')
    data['node_groups'] = _format_node_groups_list(data['node_groups'])
    data['anti_affinity'] = osc_utils.format_list(data['anti_affinity'])


def _prepare_health_checks(data):
    additional_data = {}
    ver = data.get('verification', {})
    additional_fields = ['verification_status']
    additional_data['verification_status'] = ver.get('status', 'UNKNOWN')
    for check in ver.get('checks', []):
        row_name = "Health check (%s)" % check['name']
        additional_data[row_name] = check['status']
        additional_fields.append(row_name)
    return additional_data, additional_fields


def _get_plugin_version(cluster_template, client):
    ct = utils.get_resource(client.cluster_templates, cluster_template)
    return ct.plugin_name, ct.hadoop_version, ct.id


class CreateCluster(show.ShowOne):
    """Creates cluster"""

    log = logging.getLogger(__name__ + ".CreateCluster")

    def get_parser(self, prog_name):
        parser = super(CreateCluster, self).get_parser(prog_name)

        parser.add_argument(
            '--name',
            metavar="<name>",
            help="Name of the cluster [REQUIRED if JSON is not provided]",
        )
        parser.add_argument(
            '--cluster-template',
            metavar="<cluster-template>",
            help="Cluster template name or ID [REQUIRED if JSON is not "
                 "provided]"
        )
        parser.add_argument(
            '--image',
            metavar="<image>",
            help='Image that will be used for cluster deployment (Name or ID) '
                 '[REQUIRED if JSON is not provided]'
        )
        parser.add_argument(
            '--description',
            metavar="<description>",
            help='Description of the cluster'
        )
        parser.add_argument(
            '--user-keypair',
            metavar="<keypair>",
            help='User keypair to get acces to VMs after cluster creation'
        )
        parser.add_argument(
            '--neutron-network',
            metavar="<network>",
            help='Instances of the cluster will get fixed IP addresses in '
                 'this network. (Name or ID should be provided)'
        )
        parser.add_argument(
            '--count',
            metavar="<count>",
            type=int,
            help='Number of clusters to be created'
        )
        parser.add_argument(
            '--public',
            action='store_true',
            default=False,
            help='Make the cluster public (Visible from other tenants)',
        )
        parser.add_argument(
            '--protected',
            action='store_true',
            default=False,
            help='Make the cluster protected',
        )
        parser.add_argument(
            '--transient',
            action='store_true',
            default=False,
            help='Create transient cluster',
        )
        parser.add_argument(
            '--json',
            metavar='<filename>',
            help='JSON representation of the cluster. Other '
                 'arguments (except for --wait) will not be taken into '
                 'account if this one is provided'
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            default=False,
            help='Wait for the cluster creation to complete',
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing
        network_client = self.app.client_manager.network

        if parsed_args.json:
            blob = osc_utils.read_blob_file_contents(parsed_args.json)
            try:
                template = json.loads(blob)
            except ValueError as e:
                raise exceptions.CommandError(
                    'An error occurred when reading '
                    'template from file %s: %s' % (parsed_args.json, e))

            if 'neutron_management_network' in template:
                template['net_id'] = template.pop('neutron_management_network')

            if 'count' in template:
                parsed_args.count = template['count']

            data = client.clusters.create(**template).to_dict()
        else:
            if not parsed_args.name or not parsed_args.cluster_template \
                    or not parsed_args.image:
                raise exceptions.CommandError(
                    'At least --name , --cluster-template, --image arguments '
                    'should be specified or json template should be provided '
                    'with --json argument')

            plugin, version, template_id = _get_plugin_version(
                parsed_args.cluster_template, client)

            image_id = utils.get_resource_id(client.images, parsed_args.image)

            net_id = (network_client.find_network(
                parsed_args.neutron_network, ignore_missing=False).id if
                parsed_args.neutron_network else None)

            data = client.clusters.create(
                name=parsed_args.name,
                plugin_name=plugin,
                hadoop_version=version,
                cluster_template_id=template_id,
                default_image_id=image_id,
                description=parsed_args.description,
                is_transient=parsed_args.transient,
                user_keypair_id=parsed_args.user_keypair,
                net_id=net_id,
                count=parsed_args.count,
                is_public=parsed_args.public,
                is_protected=parsed_args.protected
            ).to_dict()
        if parsed_args.count and parsed_args.count > 1:
            clusters = [
                utils.get_resource(client.clusters, id)
                for id in data['clusters']]

            if parsed_args.wait:
                for cluster in clusters:
                    if not osc_utils.wait_for_status(
                            client.clusters.get, cluster.id):
                        self.log.error(
                            'Error occurred during cluster creation: %s' %
                            data['id'])

            data = {}
            for cluster in clusters:
                data[cluster.name] = cluster.id

        else:
            if parsed_args.wait:
                if not osc_utils.wait_for_status(
                        client.clusters.get, data['id']):
                    self.log.error(
                        'Error occurred during cluster creation: %s' %
                        data['id'])
                data = client.clusters.get(data['id']).to_dict()
            _format_cluster_output(data)
            data = utils.prepare_data(data, CLUSTER_FIELDS)

        return self.dict2columns(data)


class ListClusters(lister.Lister):
    """Lists clusters"""

    log = logging.getLogger(__name__ + ".ListClusters")

    def get_parser(self, prog_name):
        parser = super(ListClusters, self).get_parser(prog_name)
        parser.add_argument(
            '--long',
            action='store_true',
            default=False,
            help='List additional fields in output',
        )
        parser.add_argument(
            '--plugin',
            metavar="<plugin>",
            help="List clusters with specific plugin"
        )

        parser.add_argument(
            '--version',
            metavar="<version>",
            help="List clusters with specific version of the "
                 "plugin"
        )

        parser.add_argument(
            '--name',
            metavar="<name-substring>",
            help="List clusters with specific substring in the name"
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing
        search_opts = {}
        if parsed_args.plugin:
            search_opts['plugin_name'] = parsed_args.plugin
        if parsed_args.version:
            search_opts['hadoop_version'] = parsed_args.version

        data = client.clusters.list(search_opts=search_opts)

        if parsed_args.name:
            data = utils.get_by_name_substring(data, parsed_args.name)

        if parsed_args.long:
            columns = ('name', 'id', 'plugin_name', 'hadoop_version',
                       'status', 'description', 'default_image_id')
            column_headers = utils.prepare_column_headers(
                columns, {'hadoop_version': 'version',
                          'default_image_id': 'image'})

        else:
            columns = ('name', 'id', 'plugin_name', 'hadoop_version', 'status')
            column_headers = utils.prepare_column_headers(
                columns, {'hadoop_version': 'version',
                          'default_image_id': 'image'})
        return (
            column_headers,
            (osc_utils.get_item_properties(
                s,
                columns
            ) for s in data)
        )


class ShowCluster(show.ShowOne):
    """Display cluster details"""

    log = logging.getLogger(__name__ + ".ShowCluster")

    def get_parser(self, prog_name):
        parser = super(ShowCluster, self).get_parser(prog_name)
        parser.add_argument(
            "cluster",
            metavar="<cluster>",
            help="Name or id of the cluster to display",
        )
        parser.add_argument(
            '--verification',
            action='store_true',
            default=False,
            help='List additional fields for verifications',
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing

        data = utils.get_resource(
            client.clusters, parsed_args.cluster).to_dict()

        _format_cluster_output(data)
        fields = []
        if parsed_args.verification:
            ver_data, fields = _prepare_health_checks(data)
            data.update(ver_data)
        fields.extend(CLUSTER_FIELDS)
        data = utils.prepare_data(data, fields)

        return self.dict2columns(data)


class DeleteCluster(command.Command):
    """Deletes cluster"""

    log = logging.getLogger(__name__ + ".DeleteCluster")

    def get_parser(self, prog_name):
        parser = super(DeleteCluster, self).get_parser(prog_name)
        parser.add_argument(
            "cluster",
            metavar="<cluster>",
            nargs="+",
            help="Name(s) or id(s) of the cluster(s) to delete",
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            default=False,
            help='Wait for the cluster(s) delete to complete',
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing
        clusters = []
        for cluster in parsed_args.cluster:
            cluster_id = utils.get_resource_id(
                client.clusters, cluster)
            client.clusters.delete(cluster_id)
            clusters.append((cluster_id, cluster))
            sys.stdout.write(
                'Cluster "{cluster}" deletion has been started.\n'.format(
                    cluster=cluster))
        if parsed_args.wait:
            for cluster_id, cluster_arg in clusters:
                if not utils.wait_for_delete(client.clusters, cluster_id):
                    self.log.error(
                        'Error occurred during cluster deleting: %s' %
                        cluster_id)
                else:
                    sys.stdout.write(
                        'Cluster "{cluster}" has been removed '
                        'successfully.\n'.format(cluster=cluster_arg))


class UpdateCluster(show.ShowOne):
    """Updates cluster"""

    log = logging.getLogger(__name__ + ".UpdateCluster")

    def get_parser(self, prog_name):
        parser = super(UpdateCluster, self).get_parser(prog_name)

        parser.add_argument(
            'cluster',
            metavar="<cluster>",
            help="Name or ID of the cluster",
        )
        parser.add_argument(
            '--name',
            metavar="<name>",
            help="New name of the cluster",
        )
        parser.add_argument(
            '--description',
            metavar="<description>",
            help='Description of the cluster'
        )
        parser.add_argument(
            '--shares',
            metavar="<filename>",
            help='JSON representation of the manila shares'
        )
        public = parser.add_mutually_exclusive_group()
        public.add_argument(
            '--public',
            action='store_true',
            help='Make the cluster public '
                 '(Visible from other tenants)',
            dest='is_public'
        )
        public.add_argument(
            '--private',
            action='store_false',
            help='Make the cluster private '
                 '(Visible only from this tenant)',
            dest='is_public'
        )
        protected = parser.add_mutually_exclusive_group()
        protected.add_argument(
            '--protected',
            action='store_true',
            help='Make the cluster protected',
            dest='is_protected'
        )
        protected.add_argument(
            '--unprotected',
            action='store_false',
            help='Make the cluster unprotected',
            dest='is_protected'
        )
        parser.set_defaults(is_public=None, is_protected=None)

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing

        cluster_id = utils.get_resource_id(
            client.clusters, parsed_args.cluster)

        shares = None
        if parsed_args.shares:
            blob = osc_utils.read_blob_file_contents(parsed_args.shares)
            try:
                shares = json.loads(blob)
            except ValueError as e:
                raise exceptions.CommandError(
                    'An error occurred when reading '
                    'shares from file %s: %s' % (parsed_args.shares, e))

        update_dict = utils.create_dict_from_kwargs(
            name=parsed_args.name,
            description=parsed_args.description,
            is_public=parsed_args.is_public,
            is_protected=parsed_args.is_protected,
            shares=shares
        )
        data = client.clusters.update(cluster_id, **update_dict).cluster

        _format_cluster_output(data)
        data = utils.prepare_data(data, CLUSTER_FIELDS)

        return self.dict2columns(data)


class ScaleCluster(show.ShowOne):
    """Scales cluster"""

    log = logging.getLogger(__name__ + ".ScaleCluster")

    def get_parser(self, prog_name):
        parser = super(ScaleCluster, self).get_parser(prog_name)

        parser.add_argument(
            'cluster',
            metavar="<cluster>",
            help="Name or ID of the cluster",
        )
        parser.add_argument(
            '--instances',
            nargs='+',
            metavar='<node-group-template:instances_count>',
            help='Node group templates and number of their instances to be '
                 'scale to [REQUIRED if JSON is not provided]'
        )
        parser.add_argument(
            '--json',
            metavar='<filename>',
            help='JSON representation of the cluster scale object. Other '
                 'arguments (except for --wait) will not be taken into '
                 'account if this one is provided'
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            default=False,
            help='Wait for the cluster scale to complete',
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing

        cluster = utils.get_resource(
            client.clusters, parsed_args.cluster)

        if parsed_args.json:
            blob = osc_utils.read_blob_file_contents(parsed_args.json)
            try:
                template = json.loads(blob)
            except ValueError as e:
                raise exceptions.CommandError(
                    'An error occurred when reading '
                    'template from file %s: %s' % (parsed_args.json, e))

            data = client.clusters.scale(cluster.id, template).to_dict()
        else:
            scale_object = {
                "add_node_groups": [],
                "resize_node_groups": []
            }
            scale_node_groups = dict(
                map(lambda x: x.split(':', 1), parsed_args.instances))
            cluster_ng_map = {
                ng['node_group_template_id']: ng['name'] for ng
                in cluster.node_groups}
            for name, count in scale_node_groups.items():
                ngt = utils.get_resource(client.node_group_templates, name)
                if ngt.id in cluster_ng_map:
                    scale_object["resize_node_groups"].append({
                        "name": cluster_ng_map[ngt.id],
                        "count": int(count)
                    })
                else:
                    scale_object["add_node_groups"].append({
                        "node_group_template_id": ngt.id,
                        "name": ngt.name,
                        "count": int(count)
                    })
            if not scale_object['add_node_groups']:
                del scale_object['add_node_groups']
            if not scale_object['resize_node_groups']:
                del scale_object['resize_node_groups']

            data = client.clusters.scale(cluster.id, scale_object).cluster

        sys.stdout.write(
            'Cluster "{cluster}" scaling has been started.\n'.format(
                cluster=parsed_args.cluster))
        if parsed_args.wait:
            if not osc_utils.wait_for_status(
                    client.clusters.get, data['id']):
                self.log.error(
                    'Error occurred during cluster scaling: %s' %
                    cluster.id)
            data = client.clusters.get(cluster.id).to_dict()

        _format_cluster_output(data)
        data = utils.prepare_data(data, CLUSTER_FIELDS)

        return self.dict2columns(data)


class VerificationUpdateCluster(show.ShowOne):
    """Updates cluster verifications"""

    log = logging.getLogger(__name__ + ".VerificationUpdateCluster")

    def get_parser(self, prog_name):
        parser = super(VerificationUpdateCluster, self).get_parser(prog_name)

        parser.add_argument(
            'cluster',
            metavar="<cluster>",
            help="Name or ID of the cluster",
        )
        status = parser.add_mutually_exclusive_group(required=True)
        status.add_argument(
            '--start',
            action='store_const',
            const='START',
            help='Start health verification for the cluster',
            dest='status'
        )
        status.add_argument(
            '--show',
            help='Show health of the cluster',
            action='store_true'
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.data_processing

        if parsed_args.show:
            data = utils.get_resource(
                client.clusters, parsed_args.cluster).to_dict()
            ver_data, ver_fields = _prepare_health_checks(data)
            data = utils.prepare_data(ver_data, ver_fields)
            return self.dict2columns(data)
        else:
            cluster_id = utils.get_resource_id(
                client.clusters, parsed_args.cluster)
            client.clusters.verification_update(
                cluster_id, parsed_args.status)
            if parsed_args.status == 'START':
                print_status = 'started'
            sys.stdout.write(
                'Cluster "{cluster}" health verification has been '
                '{status}.'.format(cluster=parsed_args.cluster,
                                   status=print_status))

            return {}, {}



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_utils import importutils


class UnsupportedVersion(Exception):
    """Indication for using an unsupported version of the API.

    Indicates that the user is trying to use an unsupported
    version of the API.
    """
    pass


def get_client_class(version):
    version_map = {
        '1.0': 'saharaclient.api.client.Client',
        '1.1': 'saharaclient.api.client.Client',
    }
    try:
        client_path = version_map[str(version)]
    except (KeyError, ValueError):
        supported_versions = ', '.join(version_map.keys())
        msg = ("Invalid client version '%(version)s'; must be one of: "
               "%(versions)s") % {'version': version,
                                  'versions': supported_versions}
        raise UnsupportedVersion(msg)

    return importutils.import_class(client_path)


def Client(version, *args, **kwargs):
    client_class = get_client_class(version)
    return client_class(*args, **kwargs)



# Copyright (c) 2014 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import shlex
import subprocess

from saharaclient.tests.integration.configs import config as cfg

cfg = cfg.ITConfig()

LOG = logging.getLogger(__name__)


# This is modeled after the client interface in tempest cli tests.2
class CommandBase(object):
    def sahara(self, action, flags='', params='', fail_ok=False):
        return self.cmd_with_bypass('sahara', action, flags, params, fail_ok)

    def cmd_with_bypass(self, cmd, action, flags='', params='', fail_ok=False):
        if cfg.common_config['BYPASS_URL']:
            bypass = '--bypass-url %s' % cfg.common_config['BYPASS_URL']
            flags = bypass + ' ' + flags
        return self.cmd_with_auth(cmd, action, flags, params, fail_ok)

    def cmd_with_auth(self, cmd, action, flags='', params='', fail_ok=False):
        """Executes given command with auth attributes appended."""
        creds = ('--os-username %s --os-tenant-name %s --os-password %s '
                 '--os-auth-url %s ' %
                 (cfg.common_config['OS_USERNAME'],
                  cfg.common_config['OS_TENANT_NAME'],
                  cfg.common_config['OS_PASSWORD'],
                  cfg.common_config['OS_AUTH_URL']))
        flags = creds + ' ' + flags
        return self.cmd(cmd, action, flags, params, fail_ok)

    def cmd(self, cmd, action, flags='', params='', fail_ok=False,
            merge_stderr=False):
        """Executes specified command for the given action."""
        cmd = ' '.join([cmd, flags, action, params])
        LOG.info("running: '%s'", cmd)
        cmd_str = cmd
        cmd = shlex.split(cmd)
        result = ''
        result_err = ''
        try:
            stdout = subprocess.PIPE
            stderr = subprocess.STDOUT if merge_stderr else subprocess.PIPE
            proc = subprocess.Popen(
                cmd, stdout=stdout, stderr=stderr)
            result, result_err = proc.communicate()
            if not fail_ok and proc.returncode != 0:
                raise CommandFailed(proc.returncode,
                                    cmd,
                                    result)
        finally:
            LOG.debug('output of %s:\n%s', cmd_str, result)
            if not merge_stderr and result_err:
                LOG.debug('error output of %s:\n%s', cmd_str, result_err)
        return result


class CommandFailed(subprocess.CalledProcessError):
    # adds output attribute for python2.6
    def __init__(self, returncode, cmd, output):
        super(CommandFailed, self).__init__(returncode, cmd)
        self.output = output


class CLICommands(CommandBase):
    def register_image(self, image_id, username='', description=''):
        params = '--id %s' % image_id
        if username:
            params += ' --username %s' % username
        if description:
            params += ' --description %s' % description
        return self.sahara('image-register', params=params)

    def unregister_image(self, id):
        params = '--id %s' % id
        return self.sahara('image-unregister', params=params)

    def tag_image(self, id, tag):
        params = '--id %s --tag %s' % (id, tag)
        return self.sahara('image-add-tag', params=params)

    def node_group_template_create(self, filename):
        params = '--json %s' % filename
        return self.sahara('node-group-template-create', params=params)

    def node_group_template_delete(self, id):
        params = '--id %s' % id
        return self.sahara('node-group-template-delete', params=params)

    def cluster_template_create(self, filename):
        params = '--json %s' % filename
        return self.sahara('cluster-template-create', params=params)

    def cluster_template_delete(self, id):
        params = '--id %s' % id
        return self.sahara('cluster-template-delete', params=params)

    def cluster_create(self, filename):
        params = '--json %s' % filename
        return self.sahara('cluster-create', params=params)

    def cluster_delete(self, id):
        params = '--id %s' % id
        return self.sahara('cluster-delete', params=params)

    def job_binary_create(self, name, url, desc='', username='', password=''):
        params = '--name %s --url %s' % (name, url)
        if desc:
            params += ' --description %s' % desc
        if username:
            params += ' --user %s' % username
        if password:
            params += ' --password %s' % password
        return self.sahara('job-binary-create', params=params)

    def job_binary_delete(self, id):
        params = '--id %s' % id
        return self.sahara('job-binary-delete', params=params)

    def job_binary_data_create(self, fname):
        params = '--file %s' % fname
        return self.sahara('job-binary-data-create', params=params)

    def job_binary_data_delete(self, id):
        params = '--id %s' % id
        return self.sahara('job-binary-data-delete', params=params)

    def job_template_create(self, filename):
        params = '--json %s' % (filename)
        return self.sahara('job-template-create', params=params)

    def job_template_delete(self, id):
        params = '--id %s' % id
        return self.sahara('job-template-delete', params=params)

    def job_create(self, job_template_id, filename):
        params = '--job-template %s --json %s' % (job_template_id, filename)
        return self.sahara('job-create', params=params)

    def job_delete(self, id):
        params = '--id %s' % id
        return self.sahara('job-delete', params=params)

    def data_source_create(self, name, datatype, url,
                           desc='', username='', password=''):
        params = '--name %s --type %s --url %s' % (name, datatype, url)
        if desc:
            params += ' --description %s' % desc
        if username:
            params += ' --user %s' % username
        if password:
            params += ' --password %s' % password
        return self.sahara('data-source-create', params=params)

    def data_source_delete(self, id):
        params = '--id %s' % id
        return self.sahara('data-source-delete', params=params)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from saharaclient.api import base


class JobExecution(base.Resource):
    resource_name = 'JobExecution'


class JobExecutionsManager(base.ResourceManager):
    resource_class = JobExecution
    NotUpdated = base.NotUpdated()

    def list(self, search_opts=None):
        """Get a list of Job Executions."""
        query = base.get_query_string(search_opts)
        return self._list('/job-executions%s' % query, 'job_executions')

    def get(self, obj_id):
        """Get information about a Job Execution."""
        return self._get('/job-executions/%s' % obj_id, 'job_execution')

    def delete(self, obj_id):
        """Delete a Job Execution."""
        self._delete('/job-executions/%s' % obj_id)

    def create(self, job_id, cluster_id, input_id=None,
               output_id=None, configs=None, interface=None, is_public=None,
               is_protected=None):
        """Launch a Job."""

        url = "/jobs/%s/execute" % job_id
        data = {
            "cluster_id": cluster_id,
        }

        self._copy_if_defined(data, input_id=input_id, output_id=output_id,
                              job_configs=configs, interface=interface,
                              is_public=is_public, is_protected=is_protected)

        return self._create(url, data, 'job_execution')

    def update(self, obj_id, is_public=NotUpdated, is_protected=NotUpdated):
        """Update a Job Execution."""

        data = {}
        self._copy_if_updated(data, is_public=is_public,
                              is_protected=is_protected)
        return self._patch('/job-executions/%s' % obj_id, data)



# Copyright (c) 2014 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import random
import re
import string
import tempfile
import time

import six

import saharaclient.api.client as client
from saharaclient.tests.integration.configs import config as cfg
from swiftclient import client as swift_client

cfg = cfg.ITConfig()
common = cfg.common_config

# cluster status
CLUSTER_STATUS_ACTIVE = "Active"
CLUSTER_STATUS_ERROR = "Error"


class Utils(object):
    def __init__(self):
        self.client = client.Client(username=common['OS_USERNAME'],
                                    api_key=common['OS_PASSWORD'],
                                    auth_url=common['OS_AUTH_URL'],
                                    project_name=common['OS_PROJECT_NAME'])

    def find_image_by_id(self, id):
        return self.client.images.get(id)

    def find_cluster_by_id(self, id):
        return self.client.clusters.get(id)

    def find_node_group_template_by_id(self, id):
        return self.client.node_group_templates.get(id)

    def find_cluster_template_by_id(self, id):
        return self.client.cluster_templates.get(id)

    def find_object_by_field(self, val, obj_list, field="name"):
        for obj in obj_list:
            if getattr(obj, field) == val:
                return obj

    def find_node_group_template_by_name(self, name):
        return self.find_object_by_field(
            name,
            self.client.node_group_templates.list())

    def find_cluster_template_by_name(self, name):
        return self.find_object_by_field(name,
                                         self.client.cluster_templates.list())

    def find_cluster_by_name(self, name):
        return self.find_object_by_field(name,
                                         self.client.clusters.list())

    def find_job_by_id(self, id):
        return self.client.job_executions.get(id)

    def find_job_binary_by_id(self, id):
        return self.client.job_binaries.get(id)

    def find_job_template_by_id(self, id):
        return self.client.jobs.get(id)

    def find_data_source_by_id(self, id):
        return self.client.data_sources.get(id)

    def find_binary_internal_by_id(self, id):
        return self.client.job_binary_internals.get(id)

    def find_job_binary_by_name(self, name):
        return self.find_object_by_field(name,
                                         self.client.job_binaries.list())

    def find_job_template_by_name(self, name):
        return self.find_object_by_field(name,
                                         self.client.jobs.list())

    def find_data_source_by_name(self, name):
        return self.find_object_by_field(name,
                                         self.client.data_sources.list())

    def find_job_by_job_template_id(self, id):
        return self.find_object_by_field(id,
                                         self.client.job_executions.list(),
                                         "job_id")

    def find_binary_internal_id(self, output):
        pattern = '\|\s*%s\s*\|\s*%s'  # match '| id | name'
        internals = [(i.id,
                      i.name) for i in self.client.job_binary_internals.list()]
        for i in internals:
            prog = re.compile(pattern % i)
            if prog.search(output):
                return i[0]

    def generate_json_file(self, temp):
        f = tempfile.NamedTemporaryFile(delete=True)
        f.write(json.dumps(temp))
        f.flush()
        return f

    def poll_cluster_state(self, id):
        cluster = self.client.clusters.get(id)
        # TODO(tmckay): this should use timeutils but we need
        # to add it to openstack/common
        timeout = common['CLUSTER_CREATION_TIMEOUT'] * 60
        while str(cluster.status) != CLUSTER_STATUS_ACTIVE:
            if str(cluster.status) == CLUSTER_STATUS_ERROR or timeout <= 0:
                break
            time.sleep(10)
            timeout -= 10
            cluster = self.client.clusters.get(id)
        return str(cluster.status)

    def poll_job_execution(self, id):
        # TODO(tmckay): this should use timeutils but we need
        # to add it to openstack/common
        timeout = common['JOB_LAUNCH_TIMEOUT'] * 60
        status = self.client.job_executions.get(id).info['status']
        while status != 'SUCCEEDED':
            if status == 'KILLED' or timeout <= 0:
                break
            time.sleep(10)
            timeout -= 10
            status = self.client.job_executions.get(id).info['status']
        return status


class SwiftUtils(object):
    def __init__(self):
        self.containers = []
        self.client = swift_client.Connection(
            authurl=common['OS_AUTH_URL'],
            user=common['OS_USERNAME'],
            key=common['OS_PASSWORD'],
            tenant_name=common['OS_TENANT_NAME'],
            auth_version=common['SWIFT_AUTH_VERSION'])

    def create_container(self, marker):
        container_name = 'cli-test-%s' % marker
        self.client.put_container(container_name)
        self.containers.append(container_name)
        return container_name

    def generate_input(self, container_name, input_name):
        self.client.put_object(
            container_name, input_name, ''.join(
                random.choice(':' + ' ' + '\n' + string.ascii_lowercase)
                for x in range(10000)
            )
        )

    def upload(self, container_name, obj_name, data):
        self.client.put_object(container_name, obj_name, data)

    def delete_containers(self):
        for container in self.containers:
            objects = [obj['name'] for obj in (
                self.client.get_container(container)[1])]
            for obj in objects:
                self.client.delete_object(container, obj)
            self.client.delete_container(container)


class AssertionWrappers(object):
    def check_dict_elems_in_obj(self, d, obj, exclude=[]):
        for key, val in six.iteritems(d):
            if key not in exclude:
                self.assertEqual(val, getattr(obj, key))

    def check_dict_is_subset(self, dict1, dict2):
        # There is an assert for this in Python 2.7 but not 2.6
        self.assertTrue(all(
            k in dict2 and dict2[k] == v for k, v in six.iteritems(dict1)))



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from saharaclient.api import base


class Job(base.Resource):
    resource_name = 'Job'


class JobsManager(base.ResourceManager):
    resource_class = Job
    NotUpdated = base.NotUpdated()

    def create(self, name, type, mains=None, libs=None, description=None,
               interface=None, is_public=None, is_protected=None):
        """Create a Job."""
        data = {
            'name': name,
            'type': type
        }

        self._copy_if_defined(data, description=description, mains=mains,
                              libs=libs, interface=interface,
                              is_public=is_public, is_protected=is_protected)

        return self._create('/jobs', data, 'job')

    def list(self, search_opts=None):
        """Get a list of Jobs."""
        query = base.get_query_string(search_opts)
        return self._list('/jobs%s' % query, 'jobs')

    def get(self, job_id):
        """Get information about a Job"""
        return self._get('/jobs/%s' % job_id, 'job')

    def get_configs(self, job_type):
        """Get config hints for a specified Job type."""
        return self._get('/jobs/config-hints/%s' % job_type)

    def delete(self, job_id):
        """Delete a Job"""
        self._delete('/jobs/%s' % job_id)

    def update(self, job_id, name=NotUpdated, description=NotUpdated,
               is_public=NotUpdated, is_protected=NotUpdated):
        """Update a Job."""

        data = {}
        self._copy_if_updated(data, name=name, description=description,
                              is_public=is_public, is_protected=is_protected)

        return self._patch('/jobs/%s' % job_id, data)



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

from saharaclient.api import job_binary_internals as jbi
from saharaclient.tests.unit import base


class JobBinaryInternalTest(base.BaseTestCase):
    body = {
        'name': 'name',
        'datasize': '123',
        'id': 'id'
    }

    def test_create_job_binary_internal(self):
        url = self.URL + '/job-binary-internals/name'
        self.responses.put(url, status_code=202,
                           json={'job_binary_internal': self.body})

        resp = self.client.job_binary_internals.create('name', 'data')

        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual('data', self.responses.last_request.body)
        self.assertIsInstance(resp, jbi.JobBinaryInternal)
        self.assertFields(self.body, resp)

    def test_job_binary_internal_list(self):
        url = self.URL + '/job-binary-internals'
        self.responses.get(url, json={'binaries': [self.body]})

        resp = self.client.job_binary_internals.list()

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp[0], jbi.JobBinaryInternal)
        self.assertFields(self.body, resp[0])

    def test_job_binary_get(self):
        url = self.URL + '/job-binary-internals/id'
        self.responses.get(url, json={'job_binary_internal': self.body})

        resp = self.client.job_binary_internals.get('id')

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp, jbi.JobBinaryInternal)
        self.assertFields(self.body, resp)

    def test_job_binary_delete(self):
        url = self.URL + '/job-binary-internals/id'
        self.responses.delete(url, status_code=204)

        self.client.job_binary_internals.delete('id')

        self.assertEqual(url, self.responses.last_request.url)

    def test_job_binary_update(self):
        url = self.URL + '/job-binary-internals/id'

        update_body = {
            'name': 'new_name'
        }

        self.responses.patch(url, status_code=202, json=update_body)

        # check that all parameters will be updated
        resp = self.client.job_binary_internals.update('id', name='new_name')

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp, jbi.JobBinaryInternal)
        self.assertEqual(update_body,
                         json.loads(self.responses.last_request.body))

        # check that parameters will not be updated
        self.client.job_binary_internals.update("id")
        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual({},
                         json.loads(self.responses.last_request.body))

        # check that all parameters will be unset
        unset_json = {
            'name': None, "is_public": None, "is_protected": None
        }

        self.client.job_binary_internals.update("id", **unset_json)
        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual(unset_json,
                         json.loads(self.responses.last_request.body))



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from saharaclient.api import images
from saharaclient.tests.unit import base

import json


class ImageTest(base.BaseTestCase):
    body = {
        'username': 'name',
        'description': 'descr'
    }

    def test_images_list(self):
        url = self.URL + '/images'
        self.responses.get(url, json={'images': [self.body]})

        resp = self.client.images.list()

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp[0], images.Image)
        self.assertFields(self.body, resp[0])

    def test_images_get(self):
        url = self.URL + '/images/id'
        self.responses.get(url, json={'image': self.body})

        resp = self.client.images.get('id')

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp, images.Image)
        self.assertFields(self.body, resp)

    def test_unregister_image(self):
        url = self.URL + '/images/id'
        self.responses.delete(url, status_code=204)

        self.client.images.unregister_image('id')

        self.assertEqual(url, self.responses.last_request.url)

    def test_update_image(self):
        url = self.URL + '/images/id'
        self.responses.post(url, json={'image': self.body}, status_code=202)

        self.client.images.update_image('id', 'name', 'descr')

        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual(self.body,
                         json.loads(self.responses.last_request.body))

    def test_update_tags(self):
        url = self.URL + '/images/id'
        tag_url = self.URL + '/images/id/tag'
        untag_url = self.URL + '/images/id/untag'

        body = self.body.copy()
        body['tags'] = ['fake', '0.1']

        self.responses.post(tag_url, json={'image': body},
                            status_code=202)
        self.responses.post(untag_url, json={'image': body},
                            status_code=202)
        self.responses.get(url, json={'image': body})

        resp = self.client.images.update_tags('id', ['username', 'tag'])
        self.assertIsInstance(resp, images.Image)
        self.assertFields(self.body, resp)

        resp = self.client.images.update_tags('id', ['username'])
        self.assertIsInstance(resp, images.Image)
        self.assertFields(self.body, resp)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class Parameter(object):
    """This bean is used for building config entries."""
    def __init__(self, config):
        self.name = config['name']
        self.description = config.get('description', "No description")
        self.required = not config['is_optional']
        self.default_value = config.get('default_value', None)
        self.initial_value = self.default_value
        self.param_type = config['config_type']
        self.priority = int(config.get('priority', 2))



# Copyright (c) 2014 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

import saharaclient.api.base as api_base
from saharaclient.tests.integration.configs import config as cfg
import saharaclient.tests.integration.tests.base as base
import saharaclient.tests.integration.tests.utils as utils

cfg = cfg.ITConfig()
common = cfg.common_config


class EDPTest(base.ITestBase):
    def setUp(self):
        super(EDPTest, self).setUp()

        self.swift_utils = utils.SwiftUtils()
        self.path = 'saharaclient/tests/integration/tests/resources/'

        self.job = None
        self.job_template = None
        self.lib_binary = None
        self.main_binary = None
        self.input_source = None
        self.output_source = None

    def tearDown(self):
        super(EDPTest, self).tearDown()
        self.swift_utils.delete_containers()

    def _create_binary(self, marker, info, url):
        # Use the marker value to distinguish the object, but
        # the name must start with a letter and include a file
        # extension.
        binary_name = '%s-%s' % (marker, info['name'])
        self.cli.job_binary_create(binary_name,
                                   url,
                                   username=common.OS_USERNAME,
                                   password=common.OS_PASSWORD)
        binary_obj = self.util.find_job_binary_by_name(binary_name)
        self.assertIsNotNone(binary_obj)
        self.assertEqual(binary_name, binary_obj.name)
        self.assertEqual(url, binary_obj.url)
        return binary_obj

    def _create_swift_binary(self, marker, container, info):
        if not info:
            return None
        self.swift_utils.upload(container, info['name'], info['data'])
        url = 'swift://%s/%s' % (container, info['name'])
        return self._create_binary(marker, info, url)

    def _create_internal_binary(self, marker, info):
        if not info:
            return None, None
        output = self.cli.job_binary_data_create(info['path'])
        id = self.util.find_binary_internal_id(output)
        url = 'internal-db://%s' % id
        return self._create_binary(marker, info, url), id

    def _create_data_source(self, name, url):
        self.cli.data_source_create(name, 'swift', url,
                                    username=common.OS_USERNAME,
                                    password=common.OS_PASSWORD)
        source = self.util.find_data_source_by_name(name)
        self.assertIsNotNone(source)
        return source

    def _binary_info(self, fname, relative_path=""):
        # Binaries need to be named, and the name must include
        # the file extension since Oozie depends on it. So, we
        # just store the filename for the name here.
        info = {'name': fname}
        if common.INTERNAL_JOB_BINARIES:
            # We will use the cli to upload the file by path
            info['path'] = self.path + relative_path + fname
        else:
            # We will upload the binary data to swift
            info['data'] = open(self.path + relative_path + fname).read()
        return info

    def edp_common(self, job_type, lib=None, main=None, configs=None,
                   add_data_sources=True, pass_data_sources_as_args=False,
                   job_interface=None, execution_interface=None):
        # Generate a new marker for this so we can keep containers separate
        # and create some input data
        job_interface = job_interface or []
        execution_interface = execution_interface or {}
        marker = "%s-%s" % (job_type.replace(".", ""), os.getpid())
        container = self.swift_utils.create_container(marker)
        self.swift_utils.generate_input(container, 'input')
        input_url = 'swift://%s.sahara/input' % container
        output_url = 'swift://%s.sahara/output' % container

        # Create binaries
        if common.INTERNAL_JOB_BINARIES:
            (self.lib_binary,
             self.lib_data_id) = self._create_internal_binary(marker, lib)
            (self.main_binary,
             self.main_data_id) = self._create_internal_binary(marker, main)
        else:
            self.lib_data_id = None
            self.main_data_id = None
            self.lib_binary = self._create_swift_binary(marker, container, lib)
            self.main_binary = self._create_swift_binary(marker,
                                                         container, main)

        # Create data sources
        if add_data_sources:
            self.input_source = self._create_data_source('input-%s' % marker,
                                                         input_url)
            self.output_source = self._create_data_source('output-%s' % marker,
                                                          output_url)
        else:
            self.input_source = self.output_source = None

        # Create a job template
        job_template_name = marker
        job_template_dict = {
            "name": job_template_name,
            "type": job_type,
            "mains": [self.main_binary.id] if (self.main_binary and
                                               self.main_binary.id) else [],
            "libs": [self.lib_binary.id] if (self.lib_binary and
                                             self.lib_binary.id) else [],
            "interface": job_interface
        }
        f = self.util.generate_json_file(job_template_dict)
        self.cli.job_template_create(f.name)
        self.job_template = self.util.find_job_template_by_name(
            job_template_name)
        self.assertIsNotNone(self.job_template)
        self.assertEqual(job_template_name, self.job_template.name)
        self.assertEqual(job_type, self.job_template.type)
        if self.lib_binary:
            self.assertEqual(1, len(self.job_template.libs))
            self.assertEqual(self.job_template.libs[0]['id'],
                             self.lib_binary.id)
        if self.main_binary:
            self.assertEqual(1, len(self.job_template.mains))
            self.assertEqual(self.job_template.mains[0]['id'],
                             self.main_binary.id)

        # Launch the job
        if pass_data_sources_as_args:
            args = [input_url, output_url]
        else:
            args = None

        job_dict = {
            "cluster_id": self.cluster.id,
            "input_id": self.input_source and (
                self.input_source.id or None),
            "output_id": self.output_source and (
                self.output_source.id or None),
            "job_configs": {"configs": configs,
                            "args": args},
            "interface": execution_interface
        }
        f = self.util.generate_json_file(job_dict)
        self.cli.job_create(self.job_template.id, f.name)

        # Find the job using the job_template_id
        self.job = self.util.find_job_by_job_template_id(self.job_template.id)
        self.assertIsNotNone(self.job)
        self.assertEqual(self.cluster.id, self.job.cluster_id)

        # poll for status
        status = self.util.poll_job_execution(self.job.id)
        self.assertEqual('SUCCEEDED', status)

        # follow up with a deletion of the stuff we made from a util function
        self.delete_job_objects()

    def pig_edp(self):
        self.edp_common('Pig',
                        lib=self._binary_info('edp-lib.jar'),
                        main=self._binary_info('edp-job.pig'))

    def mapreduce_edp(self):
        configs = {
            "mapred.mapper.class": "org.apache.oozie.example.SampleMapper",
            "mapred.reducer.class": "org.apache.oozie.example.SampleReducer"
        }
        self.edp_common('MapReduce',
                        lib=self._binary_info('edp-mapreduce.jar'),
                        configs=configs)

    def mapreduce_streaming_edp(self):
        configs = {
            "edp.streaming.mapper": "/bin/cat",
            "edp.streaming.reducer": "/usr/bin/wc"
        }
        self.edp_common('MapReduce.Streaming',
                        configs=configs)

    def java_edp(self):
        configs = {
            'fs.swift.service.sahara.password': common.OS_PASSWORD,
            'edp.java.main_class': 'org.openstack.sahara.examples.WordCount'
        }
        job_interface = [{
            "name": "Swift Username",
            "mapping_type": "configs",
            "location": "fs.swift.service.sahara.username",
            "value_type": "string",
            "required": True
        }]
        execution_interface = {"Swift Username": common.OS_USERNAME}
        self.edp_common('Java',
                        lib=self._binary_info('edp-java.jar',
                                              relative_path='edp-java/'),
                        configs=configs,
                        add_data_sources=False,
                        pass_data_sources_as_args=True,
                        job_interface=job_interface,
                        execution_interface=execution_interface)

    def run_edp_jobs(self, config):
        try:
            if not config.SKIP_JAVA_EDP_TEST:
                self.java_edp()
            if not config.SKIP_MAPREDUCE_EDP_TEST:
                self.mapreduce_edp()
            if not config.SKIP_MAPREDUCE_STREAMING_EDP_TEST:
                self.mapreduce_streaming_edp()
            if not config.SKIP_PIG_EDP_TEST:
                self.pig_edp()
        except Exception as e:
            # Something went wrong, try to clean up what might be left
            self.delete_job_objects_via_client()
            raise(e)

    def delete_job_objects(self):
        if self.job:
            self.cli.job_delete(self.job.id)
            self.assertRaises(api_base.APIException,
                              self.util.find_job_by_id,
                              self.job.id)
            self.job = None

        if self.job_template:
            self.cli.job_template_delete(self.job_template.id)
            self.assertRaises(api_base.APIException,
                              self.util.find_job_template_by_id,
                              self.job_template.id)
            self.job_template = None

        if self.lib_binary:
            self.cli.job_binary_delete(self.lib_binary.id)
            self.assertRaises(api_base.APIException,
                              self.util.find_job_binary_by_id,
                              self.lib_binary.id)
            self.lib_binary = None

        if self.lib_data_id:
            self.cli.job_binary_data_delete(self.lib_data_id)
            self.assertRaises(api_base.APIException,
                              self.util.find_binary_internal_by_id,
                              self.lib_data_id)
            self.lib_data_id = None

        if self.main_binary:
            self.cli.job_binary_delete(self.main_binary.id)
            self.assertRaises(api_base.APIException,
                              self.util.find_job_binary_by_id,
                              self.main_binary.id)
            self.main_binary = None

        if self.main_data_id:
            self.cli.job_binary_data_delete(self.main_data_id)
            self.assertRaises(api_base.APIException,
                              self.util.find_binary_internal_by_id,
                              self.main_data_id)
            self.main_data_id = None

        if self.input_source:
            self.cli.data_source_delete(self.input_source.id)
            self.assertRaises(api_base.APIException,
                              self.util.find_data_source_by_id,
                              self.input_source.id)
            self.input_source = None

        if self.output_source:
            self.cli.data_source_delete(self.output_source.id)
            self.assertRaises(api_base.APIException,
                              self.util.find_data_source_by_id,
                              self.output_source.id)
            self.output_source = None

    def delete_job_objects_via_client(self):
        try:
            if self.job:
                self.util.client.job_executions.delete(self.job.id)
                self.job = None
            if self.job_template:
                self.util.client.jobs.delete(self.job_template.id)
                self.job_template = None
            if self.lib_binary:
                self.util.client.job_binaries.delete(self.lib_binary.id)
                self.lib_binary = None
            if self.main_binary:
                self.util.client.job_binaries.delete(self.main_binary.id)
                self.main_binary = None
            if self.input_source:
                self.util.client.data_sources.delete(self.input_source.id)
                self.input_source = None
            if self.output_source:
                self.util.client.data_sources.delete(self.output_source.id)
                self.output_source = None
        except Exception:
            pass



# Copyright (c) 2014 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import testtools

import saharaclient.tests.integration.tests.clidriver as clidriver
import saharaclient.tests.integration.tests.utils as utils


class ITestBase(testtools.TestCase, utils.AssertionWrappers):
    def setUp(self):
        super(ITestBase, self).setUp()

        # The client is for readonly operations to check results
        self.util = utils.Utils()
        self.cli = clidriver.CLICommands()



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from openstackclient.tests import utils as osc_utils

from saharaclient.api import node_group_templates as api_ngt
from saharaclient.osc.v1 import node_group_templates as osc_ngt
from saharaclient.tests.unit.osc.v1 import fakes


NGT_INFO = {
    "node_processes": [
        "namenode",
        "tasktracker"
    ],
    "name": "template",
    "tenant_id": "tenant_id",
    "availability_zone": 'av_zone',
    "use_autoconfig": True,
    "hadoop_version": "0.1",
    "shares": None,
    "is_default": False,
    "description": 'description',
    "node_configs": {},
    "is_proxy_gateway": False,
    "auto_security_group": True,
    "volume_type": None,
    "volumes_size": 2,
    "volume_mount_prefix": "/volumes/disk",
    "plugin_name": "fake",
    "is_protected": False,
    "security_groups": None,
    "floating_ip_pool": "floating_pool",
    "is_public": True,
    "id": "ng_id",
    "flavor_id": "flavor_id",
    "volumes_availability_zone": None,
    "volumes_per_node": 2,
    "volume_local_to_instance": False
}


class TestNodeGroupTemplates(fakes.TestDataProcessing):
    def setUp(self):
        super(TestNodeGroupTemplates, self).setUp()
        self.ngt_mock = (
            self.app.client_manager.data_processing.node_group_templates)
        self.ngt_mock.reset_mock()


class TestCreateNodeGroupTemplate(TestNodeGroupTemplates):
    # TODO(apavlov): check for creation with --json
    def setUp(self):
        super(TestCreateNodeGroupTemplate, self).setUp()
        self.ngt_mock.create.return_value = api_ngt.NodeGroupTemplate(
            None, NGT_INFO)

        self.fl_mock = self.app.client_manager.compute.flavors
        self.fl_mock.get.return_value = mock.Mock(id='flavor_id')
        self.fl_mock.reset_mock()

        # Command to test
        self.cmd = osc_ngt.CreateNodeGroupTemplate(self.app, None)

    def test_ngt_create_minimum_options(self):
        arglist = ['--name', 'template', '--plugin', 'fake', '--version',
                   '0.1', '--processes', 'namenode', 'tasktracker',
                   '--flavor', 'flavor_id']
        verifylist = [('name', 'template'), ('plugin', 'fake'),
                      ('version', '0.1'), ('flavor', 'flavor_id'),
                      ('processes', ['namenode', 'tasktracker'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.ngt_mock.create.assert_called_once_with(
            auto_security_group=False, availability_zone=None,
            description=None, flavor_id='flavor_id', floating_ip_pool=None,
            hadoop_version='0.1', is_protected=False, is_proxy_gateway=False,
            is_public=False, name='template',
            node_processes=['namenode', 'tasktracker'], plugin_name='fake',
            security_groups=None, use_autoconfig=False,
            volume_local_to_instance=False,
            volume_type=None, volumes_availability_zone=None,
            volumes_per_node=None, volumes_size=None, shares=None,
            node_configs=None)

    def test_ngt_create_all_options(self):
        arglist = ['--name', 'template', '--plugin', 'fake', '--version',
                   '0.1', '--processes', 'namenode', 'tasktracker',
                   '--security-groups', 'secgr', '--auto-security-group',
                   '--availability-zone', 'av_zone', '--flavor', 'flavor_id',
                   '--floating-ip-pool', 'floating_pool', '--volumes-per-node',
                   '2', '--volumes-size', '2', '--volumes-type', 'type',
                   '--volumes-availability-zone', 'vavzone',
                   '--volumes-mount-prefix', '/volume/asd',
                   '--volumes-locality', '--description', 'descr',
                   '--autoconfig', '--proxy-gateway', '--public',
                   '--protected']

        verifylist = [('name', 'template'), ('plugin', 'fake'),
                      ('version', '0.1'),
                      ('processes', ['namenode', 'tasktracker']),
                      ('security_groups', ['secgr']),
                      ('auto_security_group', True),
                      ('availability_zone', 'av_zone'),
                      ('flavor', 'flavor_id'),
                      ('floating_ip_pool', 'floating_pool'),
                      ('volumes_per_node', 2), ('volumes_size', 2),
                      ('volumes_type', 'type'),
                      ('volumes_availability_zone', 'vavzone'),
                      ('volumes_mount_prefix', '/volume/asd'),
                      ('volumes_locality', True), ('description', 'descr'),
                      ('autoconfig', True), ('proxy_gateway', True),
                      ('public', True), ('protected', True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.ngt_mock.create.assert_called_once_with(
            auto_security_group=True, availability_zone='av_zone',
            description='descr', flavor_id='flavor_id',
            floating_ip_pool='floating_pool', hadoop_version='0.1',
            is_protected=True, is_proxy_gateway=True, is_public=True,
            name='template', node_processes=['namenode', 'tasktracker'],
            plugin_name='fake', security_groups=['secgr'], use_autoconfig=True,
            volume_local_to_instance=True, volume_type='type',
            volumes_availability_zone='vavzone', volumes_per_node=2,
            volumes_size=2, shares=None, node_configs=None)

        # Check that columns are correct
        expected_columns = (
            'Auto security group', 'Availability zone', 'Description',
            'Flavor id', 'Floating ip pool', 'Id', 'Is default',
            'Is protected', 'Is proxy gateway', 'Is public', 'Name',
            'Node processes', 'Plugin name', 'Security groups',
            'Use autoconfig', 'Version', 'Volume local to instance',
            'Volume mount prefix', 'Volume type', 'Volumes availability zone',
            'Volumes per node', 'Volumes size')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = (
            True, 'av_zone', 'description', 'flavor_id', 'floating_pool',
            'ng_id', False, False, False,
            True, 'template', 'namenode, tasktracker', 'fake', None, True,
            '0.1', False, '/volumes/disk', None, None, 2, 2)
        self.assertEqual(expected_data, data)


class TestListNodeGroupTemplates(TestNodeGroupTemplates):
    def setUp(self):
        super(TestListNodeGroupTemplates, self).setUp()
        self.ngt_mock.list.return_value = [api_ngt.NodeGroupTemplate(
            None, NGT_INFO)]

        # Command to test
        self.cmd = osc_ngt.ListNodeGroupTemplates(self.app, None)

    def test_ngt_list_no_options(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Name', 'Id', 'Plugin name', 'Version']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [('template', 'ng_id', 'fake', '0.1')]
        self.assertEqual(expected_data, list(data))

    def test_ngt_list_long(self):
        arglist = ['--long']
        verifylist = [('long', True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Name', 'Id', 'Plugin name', 'Version',
                            'Node processes', 'Description']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [('template', 'ng_id', 'fake', '0.1',
                          'namenode, tasktracker', 'description')]
        self.assertEqual(expected_data, list(data))

    def test_ngt_list_extra_search_opts(self):
        arglist = ['--plugin', 'fake', '--version', '0.1', '--name', 'templ']
        verifylist = [('plugin', 'fake'), ('version', '0.1'),
                      ('name', 'templ')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Name', 'Id', 'Plugin name', 'Version']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [('template', 'ng_id', 'fake', '0.1')]
        self.assertEqual(expected_data, list(data))


class TestShowNodeGroupTemplate(TestNodeGroupTemplates):
    def setUp(self):
        super(TestShowNodeGroupTemplate, self).setUp()
        self.ngt_mock.find_unique.return_value = api_ngt.NodeGroupTemplate(
            None, NGT_INFO)

        # Command to test
        self.cmd = osc_ngt.ShowNodeGroupTemplate(self.app, None)

    def test_ngt_show(self):
        arglist = ['template']
        verifylist = [('node_group_template', 'template')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.ngt_mock.find_unique.assert_called_once_with(name='template')

        # Check that columns are correct
        expected_columns = (
            'Auto security group', 'Availability zone', 'Description',
            'Flavor id', 'Floating ip pool', 'Id', 'Is default',
            'Is protected', 'Is proxy gateway', 'Is public', 'Name',
            'Node processes', 'Plugin name', 'Security groups',
            'Use autoconfig', 'Version', 'Volume local to instance',
            'Volume mount prefix', 'Volume type', 'Volumes availability zone',
            'Volumes per node', 'Volumes size')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = (
            True, 'av_zone', 'description', 'flavor_id', 'floating_pool',
            'ng_id', False, False, False, True, 'template',
            'namenode, tasktracker', 'fake', None, True, '0.1', False,
            '/volumes/disk', None, None, 2, 2)
        self.assertEqual(expected_data, data)


class TestDeleteNodeGroupTemplate(TestNodeGroupTemplates):
    def setUp(self):
        super(TestDeleteNodeGroupTemplate, self).setUp()
        self.ngt_mock.find_unique.return_value = api_ngt.NodeGroupTemplate(
            None, NGT_INFO)

        # Command to test
        self.cmd = osc_ngt.DeleteNodeGroupTemplate(self.app, None)

    def test_ngt_delete(self):
        arglist = ['template']
        verifylist = [('node_group_template', ['template'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.ngt_mock.delete.assert_called_once_with('ng_id')


class TestUpdateNodeGroupTemplate(TestNodeGroupTemplates):
    # TODO(apavlov): check for update with --json
    def setUp(self):
        super(TestUpdateNodeGroupTemplate, self).setUp()
        self.ngt_mock.find_unique.return_value = api_ngt.NodeGroupTemplate(
            None, NGT_INFO)
        self.ngt_mock.update.return_value = api_ngt.NodeGroupTemplate(
            None, NGT_INFO)

        self.fl_mock = self.app.client_manager.compute.flavors
        self.fl_mock.get.return_value = mock.Mock(id='flavor_id')
        self.fl_mock.reset_mock()

        # Command to test
        self.cmd = osc_ngt.UpdateNodeGroupTemplate(self.app, None)

    def test_ngt_update_no_options(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException, self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_ngt_update_nothing_updated(self):
        arglist = ['template']
        verifylist = [('node_group_template', 'template')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.ngt_mock.update.assert_called_once_with('ng_id')

    def test_ngt_update_all_options(self):
        arglist = ['template', '--name', 'template', '--plugin', 'fake',
                   '--version', '0.1', '--processes', 'namenode',
                   'tasktracker', '--security-groups', 'secgr',
                   '--auto-security-group-enable',
                   '--availability-zone', 'av_zone', '--flavor', 'flavor_id',
                   '--floating-ip-pool', 'floating_pool', '--volumes-per-node',
                   '2', '--volumes-size', '2', '--volumes-type', 'type',
                   '--volumes-availability-zone', 'vavzone',
                   '--volumes-mount-prefix', '/volume/asd',
                   '--volumes-locality-enable', '--description', 'descr',
                   '--autoconfig-enable', '--proxy-gateway-enable', '--public',
                   '--protected']

        verifylist = [('node_group_template', 'template'),
                      ('name', 'template'), ('plugin', 'fake'),
                      ('version', '0.1'),
                      ('processes', ['namenode', 'tasktracker']),
                      ('security_groups', ['secgr']),
                      ('use_auto_security_group', True),
                      ('availability_zone', 'av_zone'),
                      ('flavor', 'flavor_id'),
                      ('floating_ip_pool', 'floating_pool'),
                      ('volumes_per_node', 2), ('volumes_size', 2),
                      ('volumes_type', 'type'),
                      ('volumes_availability_zone', 'vavzone'),
                      ('volumes_mount_prefix', '/volume/asd'),
                      ('volume_locality', True),
                      ('description', 'descr'), ('use_autoconfig', True),
                      ('is_proxy_gateway', True),
                      ('is_public', True), ('is_protected', True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.ngt_mock.update.assert_called_once_with(
            'ng_id',
            auto_security_group=True, availability_zone='av_zone',
            description='descr', flavor_id='flavor_id',
            floating_ip_pool='floating_pool', hadoop_version='0.1',
            is_protected=True, is_proxy_gateway=True, is_public=True,
            name='template', node_processes=['namenode', 'tasktracker'],
            plugin_name='fake', security_groups=['secgr'], use_autoconfig=True,
            volume_local_to_instance=True, volume_type='type',
            volumes_availability_zone='vavzone', volumes_per_node=2,
            volumes_size=2)

        # Check that columns are correct
        expected_columns = (
            'Auto security group', 'Availability zone', 'Description',
            'Flavor id', 'Floating ip pool', 'Id', 'Is default',
            'Is protected', 'Is proxy gateway', 'Is public', 'Name',
            'Node processes', 'Plugin name', 'Security groups',
            'Use autoconfig', 'Version', 'Volume local to instance',
            'Volume mount prefix', 'Volume type', 'Volumes availability zone',
            'Volumes per node', 'Volumes size')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = (
            True, 'av_zone', 'description', 'flavor_id', 'floating_pool',
            'ng_id', False, False, False, True, 'template',
            'namenode, tasktracker', 'fake', None, True,
            '0.1', False, '/volumes/disk', None, None, 2, 2)
        self.assertEqual(expected_data, data)

    def test_ngt_update_private_unprotected(self):
        arglist = ['template', '--private', '--unprotected']
        verifylist = [('node_group_template', 'template'),
                      ('is_public', False), ('is_protected', False)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.ngt_mock.update.assert_called_once_with(
            'ng_id', is_protected=False, is_public=False)
